#!/usr/bin/python
"""
This example shows basic document generation functionality.

..  :copyright: (c) 2016 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import pylatex.config as cf
from pylatex import Document, NoEscape

lorem = """
Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere
cubilia Curae; Phasellus facilisis tortor vel imperdiet vestibulum. Vivamus et
mollis risus. Proin ut enim eu leo volutpat tristique. Vivamus quam enim,
efficitur quis turpis ac, condimentum tincidunt tellus. Praesent non tellus in
quam tempor dignissim. Sed feugiat ante id mauris vehicula, quis elementum nunc
molestie. Pellentesque a vulputate nisi, ut vulputate ex. Morbi erat eros,
aliquam in justo sed, placerat tempor mauris. In vitae velit eu lorem dapibus
consequat. Integer posuere ornare laoreet.

Donec pellentesque libero id tempor aliquam. Maecenas a diam at metus varius
rutrum vel in nisl. Maecenas a est lorem. Vivamus tristique nec eros ac
hendrerit. Vivamus imperdiet justo id lobortis luctus. Sed facilisis ipsum ut
tellus pellentesque tincidunt. Mauris libero lectus, maximus at mattis ut,
venenatis eget diam. Fusce in leo at erat varius laoreet. Mauris non ipsum
pretium, convallis purus vel, pulvinar leo. Aliquam lacinia lorem dapibus
tortor imperdiet, quis consequat diam mollis.

Praesent accumsan ultrices diam a eleifend. Vestibulum ante ipsum primis in
faucibus orci luctus et ultrices posuere cubilia Curae; Suspendisse accumsan
orci ut sodales ullamcorper. Integer bibendum elementum convallis. Praesent
accumsan at leo eget ullamcorper. Maecenas eget tempor enim. Quisque et nisl
eros.
"""


def main():
    cf.active = cf.Version1()
    doc = Document(data=NoEscape(lorem))
    doc.generate_pdf("config1_with_indent", clean_tex=False)

    cf.active = cf.Version1(indent=False)
    doc = Document(data=NoEscape(lorem))
    doc.generate_pdf("config2_without_indent", clean_tex=False)

    with cf.Version1().use():
        doc = Document(data=NoEscape(lorem))
        doc.generate_pdf("config3_with_indent_again", clean_tex=False)

    doc = Document(data=NoEscape(lorem))
    doc.generate_pdf("config4_without_indent_again", clean_tex=False)


if __name__ == "__main__":
    main()



#!/usr/bin/python
"""
This example shows the functionality of the longtable element.

It creates a sample multi-page spanning table

..  :copyright: (c) 2017 by Jarrah Gosbell
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, LongTable, MultiColumn


def genenerate_longtabu():
    geometry_options = {"margin": "2.54cm", "includeheadfoot": True}
    doc = Document(page_numbers=True, geometry_options=geometry_options)

    # Generate data table
    with doc.create(LongTable("l l l")) as data_table:
        data_table.add_hline()
        data_table.add_row(["header 1", "header 2", "header 3"])
        data_table.add_hline()
        data_table.end_table_header()
        data_table.add_hline()
        data_table.add_row((MultiColumn(3, align="r", data="Continued on Next Page"),))
        data_table.add_hline()
        data_table.end_table_footer()
        data_table.add_hline()
        data_table.add_row(
            (MultiColumn(3, align="r", data="Not Continued on Next Page"),)
        )
        data_table.add_hline()
        data_table.end_table_last_footer()
        row = ["Content1", "9", "Longer String"]
        for i in range(150):
            data_table.add_row(row)

    doc.generate_pdf("longtable", clean_tex=False)


genenerate_longtabu()



#!/usr/bin/python
"""
This example shows TikZ drawing capabilities.

..  :copyright: (c) 2017 by Bruno Morais
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import (
    Document,
    TikZ,
    TikZCoordinate,
    TikZDraw,
    TikZNode,
    TikZOptions,
    TikZUserPath,
)

if __name__ == "__main__":
    # create document
    doc = Document()

    # add our sample drawings
    with doc.create(TikZ()) as pic:
        # options for our node
        node_kwargs = {"align": "center", "minimum size": "100pt", "fill": "black!20"}

        # create our test node
        box = TikZNode(
            text="My block",
            handle="box",
            at=TikZCoordinate(0, 0),
            options=TikZOptions("draw", "rounded corners", **node_kwargs),
        )

        # add to tikzpicture
        pic.append(box)

        # draw a few paths
        pic.append(
            TikZDraw(
                [TikZCoordinate(0, -6), "rectangle", TikZCoordinate(2, -8)],
                options=TikZOptions(fill="red"),
            )
        )

        # show use of anchor, relative coordinate
        pic.append(TikZDraw([box.west, "--", "++(-1,0)"]))

        # demonstrate the use of the with syntax
        with pic.create(TikZDraw()) as path:
            # start at an anchor of the node
            path.append(box.east)

            # necessary here because 'in' is a python keyword
            path_options = {"in": 90, "out": 0}
            path.append(TikZUserPath("edge", TikZOptions("-latex", **path_options)))

            path.append(TikZCoordinate(1, 0, relative=True))

    doc.generate_pdf("tikzdraw", clean_tex=False)



#!/usr/bin/python
"""
This example shows the functionality of the PyLaTeX library.

It creates a sample report with 2 tables, one containing images and the other
containing data. It also creates a complex header with an image.

..  :copyright: (c) 2016 by Vladimir Gorovikov
    :license: MIT, see License for more details.
"""

# begin-doc-include
import os

from pylatex import (
    Document,
    Foot,
    Head,
    LargeText,
    LineBreak,
    LongTabu,
    MediumText,
    MiniPage,
    MultiColumn,
    NewPage,
    PageStyle,
    StandAloneGraphic,
    Tabu,
    Tabularx,
    TextColor,
    simple_page_number,
)
from pylatex.utils import NoEscape, bold


def generate_unique():
    geometry_options = {
        "head": "40pt",
        "margin": "0.5in",
        "bottom": "0.6in",
        "includeheadfoot": True,
    }
    doc = Document(geometry_options=geometry_options)

    # Generating first page style
    first_page = PageStyle("firstpage")

    # Header image
    with first_page.create(Head("L")) as header_left:
        with header_left.create(
            MiniPage(width=NoEscape(r"0.49\textwidth"), pos="c")
        ) as logo_wrapper:
            logo_file = os.path.join(os.path.dirname(__file__), "sample-logo.png")
            logo_wrapper.append(
                StandAloneGraphic(image_options="width=120px", filename=logo_file)
            )

    # Add document title
    with first_page.create(Head("R")) as right_header:
        with right_header.create(
            MiniPage(width=NoEscape(r"0.49\textwidth"), pos="c", align="r")
        ) as title_wrapper:
            title_wrapper.append(LargeText(bold("Bank Account Statement")))
            title_wrapper.append(LineBreak())
            title_wrapper.append(MediumText(bold("Date")))

    # Add footer
    with first_page.create(Foot("C")) as footer:
        message = "Important message please read"
        with footer.create(
            Tabularx("X X X X", width_argument=NoEscape(r"\textwidth"))
        ) as footer_table:
            footer_table.add_row(
                [MultiColumn(4, align="l", data=TextColor("blue", message))]
            )
            footer_table.add_hline(color="blue")
            footer_table.add_empty_row()

            branch_address = MiniPage(width=NoEscape(r"0.25\textwidth"), pos="t")
            branch_address.append("960 - 22nd street east")
            branch_address.append("\n")
            branch_address.append("Saskatoon, SK")

            document_details = MiniPage(
                width=NoEscape(r"0.25\textwidth"), pos="t", align="r"
            )
            document_details.append("1000")
            document_details.append(LineBreak())
            document_details.append(simple_page_number())

            footer_table.add_row(
                [branch_address, branch_address, branch_address, document_details]
            )

    doc.preamble.append(first_page)
    # End first page style

    # Add customer information
    with doc.create(Tabu("X[l] X[r]")) as first_page_table:
        customer = MiniPage(width=NoEscape(r"0.49\textwidth"), pos="h")
        customer.append("Verna Volcano")
        customer.append("\n")
        customer.append("For some Person")
        customer.append("\n")
        customer.append("Address1")
        customer.append("\n")
        customer.append("Address2")
        customer.append("\n")
        customer.append("Address3")

        # Add branch information
        branch = MiniPage(width=NoEscape(r"0.49\textwidth"), pos="t!", align="r")
        branch.append("Branch no.")
        branch.append(LineBreak())
        branch.append(bold("1181..."))
        branch.append(LineBreak())
        branch.append(bold("TIB Cheque"))

        first_page_table.add_row([customer, branch])
        first_page_table.add_empty_row()

    doc.change_document_style("firstpage")
    doc.add_color(name="lightgray", model="gray", description="0.80")

    # Add statement table
    with doc.create(
        LongTabu("X[l] X[2l] X[r] X[r] X[r]", row_height=1.5)
    ) as data_table:
        data_table.add_row(
            ["date", "description", "debits($)", "credits($)", "balance($)"],
            mapper=bold,
            color="lightgray",
        )
        data_table.add_empty_row()
        data_table.add_hline()
        row = ["2016-JUN-01", "Test", "$100", "$1000", "-$900"]
        for i in range(30):
            if (i % 2) == 0:
                data_table.add_row(row, color="lightgray")
            else:
                data_table.add_row(row)

    doc.append(NewPage())

    # Add cheque images
    with doc.create(LongTabu("X[c] X[c]")) as cheque_table:
        cheque_file = os.path.join(os.path.dirname(__file__), "chequeexample.png")
        cheque = StandAloneGraphic(cheque_file, image_options="width=200px")
        for i in range(0, 20):
            cheque_table.add_row([cheque, cheque])

    doc.generate_pdf("complex_report", clean_tex=False)


generate_unique()



#!/usr/bin/python

"""
Test to check when arguments of functions get changed.

This test calls functions with all available arguments to check whether they
still exist. An error from this file means that the public API has been
changed.
"""

import matplotlib
import numpy as np
import quantities as pq

from pylatex import (
    Axis,
    Center,
    ColumnType,
    Command,
    Description,
    Document,
    Enumerate,
    FBox,
    Figure,
    FlushLeft,
    FlushRight,
    Foot,
    FootnoteText,
    Head,
    HFill,
    HorizontalSpace,
    HugeText,
    Hyperref,
    Itemize,
    LargeText,
    LineBreak,
    LongTable,
    Marker,
    Math,
    Matrix,
    MdFramed,
    MediumText,
    MiniPage,
    MultiColumn,
    MultiRow,
    NewLine,
    NewPage,
    Package,
    PageStyle,
    Plot,
    Quantity,
    Section,
    SmallText,
    StandAloneGraphic,
    SubFigure,
    TableRowSizeError,
    Tabu,
    Tabular,
    Tabularx,
    TextBlock,
    TextColor,
    TikZ,
    TikZCoordinate,
    TikZDraw,
    TikZNode,
    TikZNodeAnchor,
    TikZOptions,
    TikZPath,
    TikZPathList,
    TikZScope,
    TikZUserPath,
    VectorName,
    VerticalSpace,
)
from pylatex.utils import (
    NoEscape,
    bold,
    dumps_list,
    escape_latex,
    fix_filename,
    italic,
    verbatim,
)

matplotlib.use("Agg")  # Not to use X server. For TravisCI.
import matplotlib.pyplot as pyplot  # noqa


def test_document():
    geometry_options = {
        "includeheadfoot": True,
        "headheight": "12pt",
        "headsep": "10pt",
        "landscape": True,
    }

    doc = Document(
        default_filepath="default_filepath",
        documentclass="article",
        fontenc="T1",
        inputenc="utf8",
        lmodern=True,
        data=None,
        page_numbers=True,
        indent=False,
        document_options=["a4paper", "12pt"],
        geometry_options=geometry_options,
    )

    repr(doc)

    doc.append("Some text.")
    doc.change_page_style(style="empty")
    doc.change_document_style(style="plain")
    doc.add_color(name="lightgray", model="gray", description="0.6")
    doc.add_color(name="abitless", model="gray", description="0.8")
    doc.set_variable(name="myVar", value="1234")
    doc.set_variable(name="myVar", value="1234")
    doc.change_length(parameter=r"\headheight", value="0.5in")

    doc.generate_tex(filepath="")
    doc.generate_pdf(filepath="", clean=True)


def test_section():
    sec = Section(title="", numbering=True, data=None)
    repr(sec)


def test_hyperref():
    hr = Hyperref(Marker("marker", "prefix"), "text")
    repr(hr)


def test_math():
    math = Math(data=None, inline=False)
    repr(math)

    vec = VectorName(name="")
    repr(vec)

    # Numpy
    m = np.matrix([[2, 3, 4], [0, 0, 1], [0, 0, 2]])

    matrix = Matrix(matrix=m, mtype="p", alignment=None)
    repr(matrix)


def test_table():
    # Tabular
    t = Tabular(table_spec="|c|c|", data=None, pos=None, width=2)

    t.add_hline(start=None, end=None)

    t.add_row((1, 2), escape=False, strict=True, mapper=[bold])
    t.add_row(1, 2, escape=False, strict=True, mapper=[bold])

    # MultiColumn/MultiRow.
    t.add_row((MultiColumn(size=2, align="|c|", data="MultiColumn"),), strict=True)

    # One multiRow-cell in that table would not be proper LaTeX,
    # so strict is set to False
    t.add_row((MultiRow(size=2, width="*", data="MultiRow"),), strict=False)

    repr(t)

    # TabularX
    tabularx = Tabularx(table_spec="X X X", width_argument=NoEscape(r"\textwidth"))
    tabularx.add_row(["test1", "test2", "test3"])

    # Long Table
    longtable = LongTable(table_spec="c c c")
    longtable.add_row(["test", "test2", "test3"])
    longtable.end_table_header()

    # Colored Tabu
    coloredtable = Tabu(table_spec="X[c] X[c]")
    coloredtable.add_row(["test", "test2"], color="gray", mapper=bold)

    # Colored Tabu with 'spread'
    coloredtable = Tabu(table_spec="X[c] X[c]", spread="1in")
    coloredtable.add_row(["test", "test2"], color="gray", mapper=bold)

    # Colored Tabu with 'to'
    coloredtable = Tabu(table_spec="X[c] X[c]", to="5in")
    coloredtable.add_row(["test", "test2"], color="gray", mapper=bold)

    # Colored Tabularx
    coloredtable = Tabularx(table_spec="X[c] X[c]")
    coloredtable.add_row(["test", "test2"], color="gray", mapper=bold)

    # Column
    column = ColumnType("R", "X", r"\raggedleft", parameters=2)
    repr(column)


def test_command():
    c = Command(command="documentclass", arguments=None, options=None, packages=None)
    repr(c)


def test_graphics():
    f = Figure(data=None, position=None)

    f.add_image(filename="", width=r"0.8\textwidth", placement=r"\centering")

    f.add_caption(caption="")
    repr(f)

    # Subfigure
    s = SubFigure(data=None, position=None, width=r"0.45\linewidth")

    s.add_image(filename="", width="r\linewidth", placement=None)

    s.add_caption(caption="")
    repr(s)

    # Matplotlib
    plot = Figure(data=None, position=None)

    x = [0, 1, 2, 3, 4, 5, 6]
    y = [15, 2, 7, 1, 5, 6, 9]

    pyplot.plot(x, y)

    plot.add_plot(width=r"0.8\textwidth", placement=r"\centering")
    plot.add_caption(caption="I am a caption.")
    repr(plot)

    # StandAloneGraphic
    stand_alone_graphic = StandAloneGraphic(
        filename="", image_options=r"width=0.8\textwidth"
    )
    repr(stand_alone_graphic)


def test_quantities():
    # Quantities
    Quantity(quantity=1 * pq.kg)
    q = Quantity(quantity=1 * pq.kg, format_cb=lambda x: str(int(x)))
    repr(q)


def test_package():
    # Package
    p = Package(name="", options=None)
    repr(p)


def test_tikz():
    # PGFPlots
    t = TikZ(data=None)
    repr(t)

    a = Axis(data=None, options=None)
    repr(a)

    p = Plot(name=None, func=None, coordinates=None, error_bar=None, options=None)
    repr(p)

    opt = TikZOptions(None)
    repr(opt)

    scope = TikZScope(data=None)
    repr(scope)

    c = TikZCoordinate.from_str("(0,0)")
    c = TikZCoordinate(x=0, y=0, relative=False)
    d = c + (0, 1)
    e = c - (0, 1)
    f = (0, 1) + c
    c.distance_to(d)
    repr(c)
    repr(d)
    repr(e)
    repr(f)

    bool(c == (1, 1))
    bool(c == TikZCoordinate(1, 1))
    bool(TikZCoordinate(1, 1, relative=True) == (1, 1))
    bool(TikZCoordinate(1, 1, relative=False) == (1, 1))
    bool(TikZCoordinate(1, 1, relative=True) == TikZCoordinate(1, 1, relative=False))

    # test expected to fail
    try:
        g = TikZCoordinate(0, 1, relative=True) + TikZCoordinate(1, 0, relative=False)
        repr(g)
        raise Exception
    except ValueError:
        pass

    a = TikZNodeAnchor(node_handle=None, anchor_name=None)
    repr(a)

    n = TikZNode(handle=None, options=None, at=None, text=None)
    repr(n)

    p = n.get_anchor_point("north")
    repr(p)

    p = n.get_anchor_point("_180")
    repr(p)

    p = n.west
    repr(p)

    up = TikZUserPath(path_type="edge", options=TikZOptions("bend right"))
    repr(up)

    pl = TikZPathList("(0, 1)", "--", "(2, 0)")
    pl.append((0.5, 0))
    repr(pl)

    # generate a failure, illegal start
    try:
        pl = TikZPathList("--", "(0, 1)")
        raise Exception
    except TypeError:
        pass

    # fail with illegal path type
    try:
        pl = TikZPathList("(0, 1)", "illegal", "(0, 2)")
        raise Exception
    except ValueError:
        pass

    # fail with path after path
    try:
        pl = TikZPathList("(0, 1)", "--", "--")
        raise Exception
    except ValueError:
        pass

    # other type of failure: illegal identifier after path
    try:
        pl = TikZPathList("(0, 1)", "--", "illegal")
        raise Exception
    except (ValueError, TypeError):
        pass

    pt = TikZPath(path=None, options=TikZOptions("->"))
    pt.append(TikZCoordinate(0, 1, relative=True))
    repr(pt)

    pt = TikZPath(path=[n.west, "edge", TikZCoordinate(0, 1, relative=True)])
    repr(pt)

    pt = TikZPath(path=pl, options=None)
    repr(pt)

    dr = TikZDraw(path=None, options=None)
    repr(dr)


def test_lists():
    # Lists
    itemize = Itemize()
    itemize.add_item(s="item")
    itemize.append("append")
    repr(itemize)

    enum = Enumerate(enumeration_symbol=r"\alph*)", options={"start": 172})
    enum.add_item(s="item")
    enum.add_item(s="item2")
    enum.append("append")
    repr(enum)

    desc = Description()
    desc.add_item(label="label", s="item")
    desc.append("append")
    repr(desc)


def test_headfoot():
    # Page styles, headers and footers
    page_style = PageStyle("NewStyle")
    page_style.change_thickness("header", "1pt")
    page_style.change_thickness("footer", "1pt")

    header = Head("C")
    header.append("append")

    footer = Foot("C")
    footer.append("append")

    page_style.append(header)
    page_style.append(footer)
    repr(header)
    repr(footer)
    repr(page_style)


def test_position():
    repr(HorizontalSpace(size="20pt", star=False))

    repr(VerticalSpace(size="20pt", star=True))

    # Test alignment environments
    center = Center()
    center.append("append")
    repr(center)

    right = FlushRight()
    right.append("append")
    repr(right)

    left = FlushLeft()
    left.append("append")
    repr(left)

    minipage = MiniPage(
        width=r"\textwidth",
        height="10pt",
        pos="t",
        align="r",
        content_pos="t",
        fontsize="Large",
    )
    minipage.append("append")
    repr(minipage)

    textblock = TextBlock(
        width="200", horizontal_pos="200", vertical_pos="200", indent=True
    )
    textblock.append("append")
    textblock.dumps()
    repr(textblock)


def test_frames():
    # Tests the framed environments

    md_framed = MdFramed()
    md_framed.append("Framed text")
    repr(md_framed)

    f_box = FBox()
    f_box.append("Fboxed text")
    repr(f_box)


def test_basic():
    # Tests the basic commands and environments
    # Basic commands
    new_page = NewPage()
    repr(new_page)

    new_line = NewLine()
    repr(new_line)

    line_break = LineBreak()
    repr(line_break)

    h_fill = HFill()
    repr(h_fill)

    # Basic environments
    huge = HugeText("Huge")
    huge.append("Huge 2")
    repr(huge)

    large = LargeText("Large")
    large.append("Large 2")
    repr(large)

    medium = MediumText("Medium")
    medium.append("Medium 2")
    repr(medium)

    small = SmallText("Small")
    small.append("Small 2")
    repr(small)

    footnote = FootnoteText("Footnote")
    footnote.append("Footnote 2")
    repr(footnote)

    text_color = TextColor("green", "GreenText")
    text_color.append("More Green Text")
    repr(text_color)


def test_utils():
    # Utils
    escape_latex(s="")

    fix_filename(path="")

    dumps_list(l=[], escape=False, token="\n")

    bold(s="")

    italic(s="")

    verbatim(s="", delimiter="|")


def test_errors():
    # Errors

    # TableRowSizeError

    # General test

    try:
        raise TableRowSizeError
    except TableRowSizeError:
        pass

    # Positive test, expected to raise Error

    t = Tabular(table_spec="|c|c|", data=None, pos=None)
    # TODO: this does not actually check if the error is raised
    try:
        # Wrong number of cells in table should raise an exception
        t.add_row((1, 2, 3), escape=False, strict=True)
    except TableRowSizeError:
        pass

    # Negative test, should not raise
    try:
        # Wrong number with strict=False should not raise an exception
        t.add_row((1, 2, 3), escape=False, strict=False)
    except TableRowSizeError:
        raise



#!/usr/bin/python
"""
This example shows matplotlib functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import matplotlib

from pylatex import Document, Figure, NoEscape, Section

matplotlib.use("Agg")  # Not to use X server. For TravisCI.
import matplotlib.pyplot as plt  # noqa


def main(fname, width, *args, **kwargs):
    geometry_options = {"right": "2cm", "left": "2cm"}
    doc = Document(fname, geometry_options=geometry_options)

    doc.append("Introduction.")

    with doc.create(Section("I am a section")):
        doc.append("Take a look at this beautiful plot:")

        with doc.create(Figure(position="htbp")) as plot:
            plot.add_plot(width=NoEscape(width), *args, **kwargs)
            plot.add_caption("I am a caption.")

        doc.append("Created using matplotlib.")

    doc.append("Conclusion.")

    doc.generate_pdf(clean_tex=False)


if __name__ == "__main__":
    x = [0, 1, 2, 3, 4, 5, 6]
    y = [15, 2, 7, 1, 5, 6, 9]

    plt.plot(x, y)

    main("matplotlib_ex-dpi", r"1\textwidth", dpi=300)
    main("matplotlib_ex-facecolor", r"0.5\textwidth", facecolor="b")



#!/usr/bin/env python

from pylatex.basic import MediumText
from pylatex.utils import dumps_list


def test_mapper():
    assert (
        dumps_list(["Test", "text"], mapper=MediumText)
        == """\\begin{large}%
Test%
\\end{large}%
\\begin{large}%
text%
\\end{large}"""
    )


if __name__ == "__main__":
    test_mapper()



# -*- coding: utf-8 -*-
"""
This module implements a class that implements a latex command.

This can be used directly or it can be inherited to make an easier interface
to it.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from reprlib import recursive_repr

from ..utils import dumps_list
from .latex_object import LatexObject


class CommandBase(LatexObject):
    """A class that represents a LaTeX command.

    The name of this class (when lowercased) will be the name of this command.
    To supply a different name set the ``_latex_name`` attribute.

    """

    def __init__(self, arguments=None, options=None, *, extra_arguments=None):
        r"""
        Args
        ----
        arguments: None, str, list or `~.Arguments`
            The main arguments of the command.
        options: None, str, list or `~.Options`
            Options of the command. These are placed in front of the arguments.
        extra_arguments: None, str, list or `~.Arguments`
            Extra arguments for the command. When these are supplied the
            options will be placed before them instead of before the normal
            arguments. This allows for a way of having one or more arguments
            before the options.

        """

        self._set_parameters(arguments, "arguments")
        self._set_parameters(options, "options")
        if extra_arguments is None:
            self.extra_arguments = None
        else:
            self._set_parameters(extra_arguments, "extra_arguments")

        super().__init__()

    def _set_parameters(self, parameters, argument_type):
        parameter_cls = Options if argument_type == "options" else Arguments

        if parameters is None:
            parameters = parameter_cls()
        elif not isinstance(parameters, parameter_cls):
            parameters = parameter_cls(parameters)

        # Pass on escaping to generated parameters
        parameters._default_escape = self._default_escape

        setattr(self, argument_type, parameters)

    def __key(self):
        """Return a hashable key, representing the command.

        Returns
        -------
        tuple
        """

        return (self.latex_name, self.arguments, self.options, self.extra_arguments)

    def __eq__(self, other):
        """Compare two commands.

        Args
        ----
        other: `~.CommandBase` instance
            The command to compare this command to


        Returns
        -------
        bool:
            If the two instances are equal
        """

        if isinstance(other, CommandBase):
            return self.__key() == other.__key()

        return False

    def __hash__(self):
        """Calculate the hash of a command.

        Returns
        -------
        int:
            The hash of the command
        """

        return hash(self.__key())

    def dumps(self):
        """Represent the command as a string in LaTeX syntax.

        Returns
        -------
        str
            The LaTeX formatted command
        """

        options = self.options.dumps()
        arguments = self.arguments.dumps()

        if self.extra_arguments is None:
            return r"\{command}{options}{arguments}".format(
                command=self.latex_name, options=options, arguments=arguments
            )

        extra_arguments = self.extra_arguments.dumps()

        return r"\{command}{arguments}{options}{extra_arguments}".format(
            command=self.latex_name,
            arguments=arguments,
            options=options,
            extra_arguments=extra_arguments,
        )


class Command(CommandBase):
    """A class that represents a LaTeX command.

    This class is meant for one-off commands. When a command of the same type
    is used multiple times it is better to subclass `.CommandBase`.
    """

    _repr_attributes_mapping = {"command": "latex_name"}

    def __init__(
        self,
        command=None,
        arguments=None,
        options=None,
        *,
        extra_arguments=None,
        packages=None
    ):
        r"""
        Args
        ----
        command: str
            Name of the command
        arguments: None, str, list or `~.Arguments`
            The main arguments of the command.
        options: None, str, list or `~.Options`
            Options of the command. These are placed in front of the arguments.
        extra_arguments: None, str, list or `~.Arguments`
            Extra arguments for the command. When these are supplied the
            options will be placed before them instead of before the normal
            arguments. This allows for a way of having one or more arguments
            before the options.
        packages: list of `~.Package` instances
            A list of the packages that this command requires

        Examples
        --------
        >>> Command('documentclass',
        >>>         options=Options('12pt', 'a4paper', 'twoside'),
        >>>         arguments='article').dumps()
        '\\documentclass[12pt,a4paper,twoside]{article}'
        >>> Command('com')
        '\\com'
        >>> Command('com', 'first')
        '\\com{first}'
        >>> Command('com', 'first', 'option')
        '\\com[option]{first}'
        >>> Command('com', 'first', 'option', 'second')
        '\\com{first}[option]{second}'

        """

        self.latex_name = command

        super().__init__(arguments, options, extra_arguments=extra_arguments)

        if packages is not None:
            self.packages |= packages


class UnsafeCommand(Command):
    """An unsafe version of the `Command` class.

    This class is meant for one-off commands that should not escape their
    arguments and options. Use this command with care and only use this when
    the arguments are hardcoded.

    When an unsafe command of the same type is used multiple times it is better
    to subclass `.CommandBase` and set the ``_default_escape`` attribute to
    false.
    """

    _default_escape = False


class Parameters(LatexObject):
    """The base class used by `~Options` and `~Arguments`.

    This class should probably never be used on its own and inhereting from it
    is only useful if a class like `~Options` or `~Arguments` is needed again.
    """

    @recursive_repr()
    def __repr__(self):
        args = [repr(a) for a in self._positional_args]
        args += ["%s=%r" % k_v for k_v in self._key_value_args.items()]
        return self.__class__.__name__ + "(" + ", ".join(args) + ")"

    def __init__(self, *args, **kwargs):
        r"""
        Args
        ----
        \*args:
            Positional parameters
        \*\*kwargs:
            Keyword parameters
        """

        if len(args) == 1 and not isinstance(args[0], str):
            if hasattr(args[0], "items") and len(kwargs) == 0:
                kwargs = args[0]  # do not just iterate over the dict keys
                args = ()
            elif hasattr(args[0], "__iter__"):
                args = args[0]

        self._positional_args = list(args)
        self._key_value_args = dict(kwargs)

        super().__init__()

    def __key(self):
        """Generate a unique hashable key representing the parameter object.

        Returns
        -------
        tuple
        """

        return tuple(self._list_args_kwargs())

    def __eq__(self, other):
        """Compare two parameters.

        Returns
        -------
        bool
        """

        return type(self) == type(other) and self.__key() == other.__key()

    def __hash__(self):
        """Generate a hash of the parameters.

        Returns
        -------
        int
        """

        return hash(self.__key())

    def _format_contents(self, prefix, separator, suffix):
        """Format the parameters.

        The formatting is done using the three arguments suplied to this
        function.

        Arguments
        ---------
        prefix: str
        separator: str
        suffix: str

        Returns
        -------
        str
        """

        params = self._list_args_kwargs()

        if len(params) <= 0:
            return ""

        string = (
            prefix + dumps_list(params, escape=self.escape, token=separator) + suffix
        )

        return string

    def _list_args_kwargs(self):
        """Make a list of strings representing al parameters.

        Returns
        -------
        list
        """

        params = []
        params.extend(self._positional_args)
        params.extend(
            ["{k}={v}".format(k=k, v=v) for k, v in self._key_value_args.items()]
        )

        return params


class Options(Parameters):
    """A class implementing LaTex options for a command.

    It supports normal positional parameters, as well as key-value pairs.
    Options are the part of a command located between the square brackets
    (``[]``). The positional parameters will be outputted in order and will
    appear before the key-value-pairs. The key value-pairs won't be outputted
    in the order in which they were entered


    Examples
    --------
    >>> args = Options('a', 'b', 'c').dumps()
    '[a,b,c]'
    >>> Options('clip', width=50, height='25em', trim='1 2 3 4').dumps()
    '[clip,trim=1 2 3 4,width=50,height=25em]'

    """

    def dumps(self):
        """Represent the parameters as a string in LaTeX syntax.

        This is to be appended to a command.

        Returns
        -------
        str
        """

        return self._format_contents("[", ",", "]")


class SpecialOptions(Options):
    r"""A class that sepparates the options with '][' instead of ','."""

    def dumps(self):
        """Represent the parameters as a string in LaTex syntax."""

        return self._format_contents("[", "][", "]")


class Arguments(Parameters):
    """A class implementing LaTex arguments for a command.

    It supports normal positional parameters, as well as key-value pairs.
    Arguments are the part of a command located between the curly braces
    (``{}``). The positional parameters will be outputted in order and will
    appear before the key-value-pairs. The key value-pairs won't be outputted
    in the order in which they were entered


    Examples
    --------
    >>> args = Arguments('a', 'b', 'c').dumps()
    '{a}{b}{c}'
    >>> args = Arguments('clip', width=50, height='25em').dumps()
    >>> args.dumps()
    '{clip}{width=50}{height=25em}'

    """

    def dumps(self):
        """Represent the parameters as a string in LaTeX syntax.

        This is to be appended to a command.

        Returns
        -------
        str
        """

        return self._format_contents("{", "}{", "}")


class SpecialArguments(Arguments):
    """A class that separates arguments with ',' instead of '}{'."""

    def dumps(self):
        """Represent the parameters as a string in LaTeX syntax.

        This is to be appended to a command.

        Returns
        -------
        str
        """

        return self._format_contents("{", ",", "}")



# -*- coding: utf-8 -*-
"""This module implements the label command and reference."""

from .base_classes import CommandBase, LatexObject
from .package import Package


def _remove_invalid_char(s):
    """Remove invalid and dangerous characters from a string."""

    s = "".join([i if ord(i) >= 32 and ord(i) < 127 else "" for i in s])
    s = s.translate(dict.fromkeys(map(ord, "&%$#_{}~^\\\n\xA0[]\":;' ")))
    return s


class Marker(LatexObject):
    """A class that represents a marker (label/ref parameter)."""

    _repr_attributes_override = [
        "name",
        "prefix",
    ]

    def __init__(self, name, prefix="", del_invalid_char=True):
        """
        Args
        ----
        name: str
            Name of the marker.
        prefix: str
            Prefix to add before the name (prefix:name).
        del_invalid_char: bool
            If True invalid and dangerous characters will be
            removed from the marker
        """

        if del_invalid_char:
            prefix = _remove_invalid_char(prefix)
            name = _remove_invalid_char(name)
        self.prefix = prefix
        self.name = name

    def __str__(self):
        return ((self.prefix + ":") if self.prefix != "" else "") + self.name

    def dumps(self):
        """Represent the Marker as a string in LaTeX syntax.

        Returns
        -------
        str

        """
        return str(self)


class RefLabelBase(CommandBase):
    """A class used as base for command that take a marker only."""

    _repr_attributes_mapping = {
        "marker": "arguments",
    }

    def __init__(self, marker):
        """
        Args
        ----
        marker: Marker
            The marker to use with the label/ref.
        """

        self.marker = marker
        super().__init__(arguments=(str(marker)))


class Label(RefLabelBase):
    """A class that represents a label."""


class Ref(RefLabelBase):
    """A class that represents a reference."""


class Pageref(RefLabelBase):
    """A class that represents a page reference."""


class Eqref(RefLabelBase):
    """A class that represent a ref to a formulae."""

    packages = [Package("amsmath")]


class Cref(RefLabelBase):
    """A class that represent a cref (not a Cref)."""

    packages = [Package("cleveref")]


class CrefUp(RefLabelBase):
    """A class that represent a Cref."""

    packages = [Package("cleveref")]
    latex_name = "Cref"


class Autoref(RefLabelBase):
    """A class that represent an autoref."""

    packages = [Package("hyperref")]


class Hyperref(CommandBase):
    """A class that represents an hyperlink to a label."""

    _repr_attributes_mapping = {
        "marker": "options",
        "text": "arguments",
    }

    packages = [Package("hyperref")]

    def __init__(self, marker, text):
        """
        Args
        ----
        marker: Marker
            The marker to use with the label/ref.
        text: str
            The text that will be shown as a link
            to the label of the same marker.
        """

        self.marker = marker
        super().__init__(options=(str(marker)), arguments=text)



#!/usr/bin/python
"""
This example shows the functionality of the Tabu and LongTabu element.

..  :copyright: (c) 2016 by Vladimir Gorovikov and Scott Wallace
    :license: MIT, see License for more details.
"""

# begin-doc-include
from random import randint

from pylatex import Center, Document, LongTabu, Tabu
from pylatex.utils import bold


def genenerate_tabus():
    geometry_options = {
        "landscape": True,
        "margin": "1.5in",
        "headheight": "20pt",
        "headsep": "10pt",
        "includeheadfoot": True,
    }
    doc = Document(page_numbers=True, geometry_options=geometry_options)

    # Generate data table with 'tight' columns
    fmt = "X[r] X[r] X[r] X[r] X[r] X[r]"
    with doc.create(LongTabu(fmt, spread="0pt")) as data_table:
        header_row1 = ["Prov", "Num", "CurBal", "IntPay", "Total", "IntR"]
        data_table.add_row(header_row1, mapper=[bold])
        data_table.add_hline()
        data_table.add_empty_row()
        data_table.end_table_header()
        data_table.add_row(["Prov", "Num", "CurBal", "IntPay", "Total", "IntR"])
        row = ["PA", "9", "$100", "%10", "$1000", "Test"]
        for i in range(40):
            data_table.add_row(row)

    with doc.create(Center()) as centered:
        with centered.create(Tabu("X[r] X[r]", spread="1in")) as data_table:
            header_row1 = ["X", "Y"]
            data_table.add_row(header_row1, mapper=[bold])
            data_table.add_hline()
            row = [randint(0, 1000), randint(0, 1000)]
            for i in range(4):
                data_table.add_row(row)

    with doc.create(Center()) as centered:
        with centered.create(Tabu("X[r] X[r]", to="4in")) as data_table:
            header_row1 = ["X", "Y"]
            data_table.add_row(header_row1, mapper=[bold])
            data_table.add_hline()
            row = [randint(0, 1000), randint(0, 1000)]
            for i in range(4):
                data_table.add_row(row)

    doc.generate_pdf("tabus", clean_tex=False)


genenerate_tabus()



from pytest import raises

from pylatex.base_classes import LatexObject


class BadObject(LatexObject):
    pass


def test_latex_object():
    with raises(TypeError):
        LatexObject()


def test_bad_object():
    with raises(TypeError):
        BadObject()



#!/usr/bin/python
"""
This example shows numpy functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import numpy as np

from pylatex import Document, Math, Matrix, Section, Subsection, VectorName

if __name__ == "__main__":
    a = np.array([[100, 10, 20]]).T

    doc = Document()
    section = Section("Numpy tests")
    subsection = Subsection("Array")

    vec = Matrix(a)
    vec_name = VectorName("a")
    math = Math(data=[vec_name, "=", vec])

    subsection.append(math)
    section.append(subsection)

    subsection = Subsection("Matrix")
    M = np.matrix([[2, 3, 4], [0, 0, 1], [0, 0, 2]])
    matrix = Matrix(M, mtype="b")
    math = Math(data=["M=", matrix])

    subsection.append(math)
    section.append(subsection)

    subsection = Subsection("Product")

    math = Math(data=["M", vec_name, "=", Matrix(M * a)])
    subsection.append(math)

    section.append(subsection)

    doc.append(section)
    doc.generate_pdf("numpy_ex", clean_tex=False)



#!/usr/bin/python
"""
This example shows the functionality of the PageHeader object.

It creates a sample page with the different types of headers and footers.

..  :copyright: (c) 2016 by Vladimir Gorovikov
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import (
    Document,
    Foot,
    Head,
    LargeText,
    LineBreak,
    MediumText,
    MiniPage,
    PageStyle,
    simple_page_number,
)
from pylatex.utils import bold


def generate_header():
    geometry_options = {"margin": "0.7in"}
    doc = Document(geometry_options=geometry_options)
    # Add document header
    header = PageStyle("header")
    # Create left header
    with header.create(Head("L")):
        header.append("Page date: ")
        header.append(LineBreak())
        header.append("R3")
    # Create center header
    with header.create(Head("C")):
        header.append("Company")
    # Create right header
    with header.create(Head("R")):
        header.append(simple_page_number())
    # Create left footer
    with header.create(Foot("L")):
        header.append("Left Footer")
    # Create center footer
    with header.create(Foot("C")):
        header.append("Center Footer")
    # Create right footer
    with header.create(Foot("R")):
        header.append("Right Footer")

    doc.preamble.append(header)
    doc.change_document_style("header")

    # Add Heading
    with doc.create(MiniPage(align="c")):
        doc.append(LargeText(bold("Title")))
        doc.append(LineBreak())
        doc.append(MediumText(bold("As at:")))

    doc.generate_pdf("header", clean_tex=False)


generate_header()



# -*- coding: utf-8 -*-
"""
This module implements classes that deal with quantities.

It converts the objects from the quantities package to latex strings that
display them using the SIunitx package. Not all units work because of name
differences between quantities and SIunitx. If you find one that doesn't work
please create a pull request that adds it to the ``UNIT_NAME_TRANSLATIONS``
dictionary.

..  :copyright: (c) 2015 by Björn Dahlgren.
    :license: MIT, see License for more details.
"""

from operator import itemgetter

from .base_classes import Command
from .package import Package
from .utils import NoEscape, escape_latex

# Translations for names used in the quantities package to ones used by SIunitx
UNIT_NAME_TRANSLATIONS = {
    "Celsius": "celsius",
    "revolutions_per_minute": "rpm",
    "v": "volt",
}


def _dimensionality_to_siunitx(dim):
    import quantities as pq

    string = ""
    items = dim.items()
    for unit, power in sorted(items, key=itemgetter(1), reverse=True):
        if power < 0:
            substring = r"\per"
            power = -power
        elif power == 0:
            continue
        else:
            substring = ""

        prefixes = [x for x in dir(pq.prefixes) if not x.startswith("_")]
        for prefix in prefixes:
            # Split unitname into prefix and actual name if possible
            if unit.name.startswith(prefix):
                substring += "\\" + prefix
                name = unit.name[len(prefix)]
                break
        else:
            # Otherwise simply use the full name
            name = unit.name

        try:
            # Check if the name is different in SIunitx
            name = UNIT_NAME_TRANSLATIONS[name]
        except KeyError:
            pass

        substring += "\\" + name

        if power > 1:
            substring += r"\tothe{" + str(power) + "}"
        string += substring
    return NoEscape(string)


class Quantity(Command):
    """A class representing quantities."""

    packages = [
        Package("siunitx", options=[NoEscape("separate-uncertainty=true")]),
        NoEscape("\\DeclareSIUnit\\rpm{rpm}"),
    ]

    def __init__(self, quantity, *, options=None, format_cb=None):
        r"""
        Args
        ----
        quantity: `quantities.quantity.Quantity`
            The quantity that should be displayed
        options: None, str, list or `~.Options`
            Options of the command. These are placed in front of the arguments.
        format_cb: callable
            A function which formats the number in the quantity. By default
            this uses `numpy.array_str`.

        Examples
        --------
        >>> import quantities as pq
        >>> speed = 3.14159265 * pq.meter / pq.second
        >>> Quantity(speed, options={'round-precision': 3,
        ...                          'round-mode': 'figures'}).dumps()
        '\\SI[round-mode=figures,round-precision=3]{3.14159265}{\meter\per\second}'

        Uncertainties are also handled:

        >>> length = pq.UncertainQuantity(16.0, pq.meter, 0.3)
        >>> width = pq.UncertainQuantity(16.0, pq.meter, 0.4)
        >>> Quantity(length*width).dumps()
        '\\SI{256.0 +- 0.5}{\meter\tothe{2}}

        Ordinary numbers are also supported:

        >>> Avogadro_constant = 6.022140857e23
        >>> Quantity(Avogadro_constant, options={'round-precision': 3}).dumps()
        '\\num[round-precision=3]{6.022e23}'

        """
        import numpy as np
        import quantities as pq

        self.quantity = quantity
        self._format_cb = format_cb

        def _format(val):
            if format_cb is None:
                try:
                    return np.array_str(val)
                except AttributeError:
                    return escape_latex(val)  # Python float and int
            else:
                return format_cb(val)

        if isinstance(quantity, pq.UncertainQuantity):
            magnitude_str = "{} +- {}".format(
                _format(quantity.magnitude), _format(quantity.uncertainty.magnitude)
            )
        elif isinstance(quantity, pq.Quantity):
            magnitude_str = _format(quantity.magnitude)

        if isinstance(quantity, (pq.UncertainQuantity, pq.Quantity)):
            unit_str = _dimensionality_to_siunitx(quantity.dimensionality)
            super().__init__(
                command="SI", arguments=(magnitude_str, unit_str), options=options
            )
        else:
            super().__init__(
                command="num", arguments=_format(quantity), options=options
            )

        self.arguments._escape = False  # dash in e.g. \num{3 +- 2}
        if self.options is not None:
            self.options._escape = False  # siunitx uses dashes in kwargs



# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with positioning.

Positions various elements on the page.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import Command, CommandBase, Environment, SpecialOptions
from .package import Package
from .utils import NoEscape


class HorizontalSpace(CommandBase):
    """Add/remove the amount of horizontal space between elements."""

    _latex_name = "hspace"

    _repr_attributes_mapping = {"size": "arguments"}

    def __init__(self, size, *, star=True):
        """
        Args
        ----
        size: str
            The amount of space to add
        star: bool
            Use the star variant of the command. Enabling this makes sure the
            space is also added where page breaking takes place.
        """

        if star:
            self.latex_name += "*"

        super().__init__(arguments=size)


class VerticalSpace(HorizontalSpace):
    """Add the user specified amount of vertical space to the document."""

    _latex_name = "vspace"


class Center(Environment):
    r"""Centered environment."""

    packages = [Package("ragged2e")]


class FlushLeft(Center):
    r"""Left-aligned environment."""


class FlushRight(Center):
    r"""Right-aligned environment."""


class MiniPage(Environment):
    r"""A class that allows the creation of minipages within document pages."""

    packages = [Package("ragged2e")]

    _repr_attributes_mapping = {
        "width": "arguments",
        "pos": "options",
        "height": "options",
        "content_pos": "options",
        "align": "options",
    }

    def __init__(
        self,
        *,
        width=NoEscape(r"\textwidth"),
        pos=None,
        height=None,
        content_pos=None,
        align=None,
        fontsize=None,
        data=None
    ):
        r"""
        Args
        ----
        width: str
            width of the minipage
        pos: str
            The vertical alignment of the minipage relative to the baseline
            (center(c), top(t), bottom(b))
        height: str
            height of the minipage
        content_pos: str
            The position of the content inside the minipage (center(c),
            bottom(b), top(t), spread(s))
        align: str
            alignment of the minibox
        fontsize: str
            The font size of the minipage
        data: str or `~.LatexObject`
            The data to place inside the MiniPage element
        """

        options = []

        if pos is not None:
            options.append(pos)

        if height is not None:
            options.append(NoEscape(height))

        if (content_pos is not None) and (pos is not None) and (height is not None):
            options.append(content_pos)

        options = SpecialOptions(*options)

        arguments = [NoEscape(str(width))]

        extra_data = []

        if align is not None:
            if align == "l":
                extra_data.append(Command(command="flushleft"))
            elif align == "c":
                extra_data.append(Command(command="centering"))
            elif align == "r":
                extra_data.append(Command(command="flushright"))

        if fontsize is not None:
            extra_data.append(Command(command=fontsize))

        if data is not None:
            if isinstance(data, list):
                data = extra_data + data
            else:
                data = extra_data + [data]
        else:
            data = extra_data

        super().__init__(arguments=arguments, options=options, data=data)


class TextBlock(Environment):
    r"""A class that represents a textblock environment.

    Make sure to set lengths of TPHorizModule and TPVertModule
    """

    _repr_attributes_mapping = {"width": "arguments"}

    packages = [Package("textpos")]

    def __init__(self, width, horizontal_pos, vertical_pos, *, indent=False, data=None):
        r"""
        Args
        ----
        width: float
            Width of the text block in the units specified by TPHorizModule
        horizontal_pos: float
            Horizontal position in units specified by the TPHorizModule
        indent: bool
            Determines whether the text block has an indent before it
        vertical_pos: float
            Vertical position in units specified by the TPVertModule
        data: str or `~.LatexObject`
            The data to place inside the TextBlock element
        """

        arguments = width
        self.horizontal_pos = horizontal_pos
        self.vertical_pos = vertical_pos

        super().__init__(arguments=arguments)

        self.append("(%s, %s)" % (str(self.horizontal_pos), str(self.vertical_pos)))

        if not indent:
            self.append(NoEscape(r"\noindent"))



#!/usr/bin/env python

"""
Test to check if configuration changes have effect.

..  :copyright: (c) 2016 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import pylatex.config as cf
from pylatex import Document


def test():
    assert type(cf.active) == cf.Default
    cf.active = cf.Version1()
    assert cf.active.indent
    assert Document()._indent

    cf.active = cf.Version1(indent=False)
    assert not cf.active.indent
    assert not Document()._indent

    with cf.Version1().use():
        assert cf.active.indent
        assert Document()._indent

    assert not cf.active.indent
    assert not Document()._indent

    with cf.active.change(indent=True):
        assert cf.active.indent
        assert Document()._indent

    assert not cf.active.indent
    assert not Document()._indent


if __name__ == "__main__":
    test()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
This example shows list functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
# Test for list structures in PyLaTeX.
# More info @ http://en.wikibooks.org/wiki/LaTeX/List_Structures
from pylatex import (
    Command,
    Description,
    Document,
    Enumerate,
    Itemize,
    NoEscape,
    Section,
)

if __name__ == "__main__":
    doc = Document()

    # create a bulleted "itemize" list like the below:
    # \begin{itemize}
    #   \item The first item
    #   \item The second item
    #   \item The third etc \ldots
    # \end{itemize}

    with doc.create(Section('"Itemize" list')):
        with doc.create(Itemize()) as itemize:
            itemize.add_item("the first item")
            itemize.add_item("the second item")
            itemize.add_item("the third etc")
            # you can append to existing items
            itemize.append(Command("ldots"))

    # create a numbered "enumerate" list like the below:
    # \begin{enumerate}[label=\alph*),start=20]
    #   \item The first item
    #   \item The second item
    #   \item The third etc \ldots
    # \end{enumerate}

    with doc.create(Section('"Enumerate" list')):
        with doc.create(
            Enumerate(enumeration_symbol=r"\alph*)", options={"start": 20})
        ) as enum:
            enum.add_item("the first item")
            enum.add_item("the second item")
            enum.add_item(NoEscape("the third etc \\ldots"))

    # create a labelled "description" list like the below:
    # \begin{description}
    #   \item[First] The first item
    #   \item[Second] The second item
    #   \item[Third] The third etc \ldots
    # \end{description}

    with doc.create(Section('"Description" list')):
        with doc.create(Description()) as desc:
            desc.add_item("First", "The first item")
            desc.add_item("Second", "The second item")
            desc.add_item("Third", NoEscape("The third etc \\ldots"))

    doc.generate_pdf("lists", clean_tex=False)



"""
A library for creating Latex files.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from . import _version
from .base_classes import Command, UnsafeCommand
from .basic import (
    FootnoteText,
    HFill,
    HugeText,
    LargeText,
    LineBreak,
    MediumText,
    NewLine,
    NewPage,
    SmallText,
    TextColor,
)
from .document import Document
from .errors import TableRowSizeError
from .figure import Figure, StandAloneGraphic, SubFigure
from .frames import FBox, MdFramed
from .headfoot import Foot, Head, PageStyle, simple_page_number
from .labelref import Autoref, Eqref, Hyperref, Label, Marker, Pageref, Ref
from .lists import Description, Enumerate, Itemize
from .math import Alignat, Math, Matrix, VectorName
from .package import Package
from .position import (
    Center,
    FlushLeft,
    FlushRight,
    HorizontalSpace,
    MiniPage,
    TextBlock,
    VerticalSpace,
)
from .quantities import Quantity
from .section import Chapter, Section, Subsection, Subsubsection
from .table import (
    ColumnType,
    LongTable,
    LongTabu,
    LongTabularx,
    MultiColumn,
    MultiRow,
    Table,
    Tabu,
    Tabular,
    Tabularx,
)
from .tikz import (
    Axis,
    Plot,
    TikZ,
    TikZCoordinate,
    TikZDraw,
    TikZNode,
    TikZNodeAnchor,
    TikZOptions,
    TikZPath,
    TikZPathList,
    TikZScope,
    TikZUserPath,
)
from .utils import NoEscape, escape_latex

__version__ = _version.get_versions()["version"]



# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with creating headers and footers.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import Command, ContainerCommand
from .package import Package
from .utils import NoEscape


class PageStyle(ContainerCommand):
    r"""Allows the creation of new page styles."""

    _latex_name = "fancypagestyle"

    packages = [Package("fancyhdr")]

    def __init__(self, name, *, header_thickness=0, footer_thickness=0, data=None):
        r"""
        Args
        ----
        name: str
            The name of the page style
        header_thickness: float
            Value to set for the line under the header
        footer_thickness: float
            Value to set for the line over the footer
        data: str or `~.LatexObject`
            The data to place inside the PageStyle
        """

        self.name = name

        super().__init__(data=data, arguments=self.name)

        self.change_thickness(element="header", thickness=header_thickness)
        self.change_thickness(element="footer", thickness=footer_thickness)

        # Clear the current header and footer
        self.append(Head())
        self.append(Foot())

    def change_thickness(self, element, thickness):
        r"""Change line thickness.

        Changes the thickness of the line under/over the header/footer
        to the specified thickness.

        Args
        ----
        element: str
            the name of the element to change thickness for: header, footer
        thickness: float
            the thickness to set the line to
        """

        if element == "header":
            self.data.append(
                Command(
                    "renewcommand",
                    arguments=[NoEscape(r"\headrulewidth"), str(thickness) + "pt"],
                )
            )
        elif element == "footer":
            self.data.append(
                Command(
                    "renewcommand",
                    arguments=[NoEscape(r"\footrulewidth"), str(thickness) + "pt"],
                )
            )


def simple_page_number():
    """Get a string containing commands to display the page number.

    Returns
    -------
    str
        The latex string that displays the page number
    """

    return NoEscape(r"Page \thepage\ of \pageref{LastPage}")


class Head(ContainerCommand):
    r"""Allows the creation of headers."""

    _latex_name = "fancyhead"

    def __init__(self, position=None, *, data=None):
        r"""
        Args
        ----
        position: str
            the headers position: L, C, R
        data: str or `~.LatexObject`
            The data to place inside the Head element
        """

        self.position = position

        super().__init__(data=data, options=position)


class Foot(Head):
    r"""Allows the creation of footers."""

    _latex_name = "fancyfoot"



# -*- coding: utf-8 -*-
"""
This module implements the classes used to show plots.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import math
import re

from .base_classes import Command, Container, Environment, LatexObject, Options
from .package import Package


class TikZOptions(Options):
    """Options class, do not escape."""

    escape = False

    def append_positional(self, option):
        """Add a new positional option."""

        self._positional_args.append(option)


class TikZ(Environment):
    """Basic TikZ container class."""

    _latex_name = "tikzpicture"
    packages = [Package("tikz")]


class Axis(Environment):
    """PGFPlots axis container class, this contains plots."""

    packages = [Package("pgfplots"), Command("pgfplotsset", "compat=newest")]

    def __init__(self, options=None, *, data=None):
        """
        Args
        ----
        options: str, list or `~.Options`
            Options to format the axis environment.
        """

        super().__init__(options=options, data=data)


class TikZScope(Environment):
    """TikZ Scope Environment."""

    _latex_name = "scope"


class TikZCoordinate(LatexObject):
    """A General Purpose Coordinate Class."""

    _coordinate_str_regex = re.compile(
        r"(\+\+)?\(\s*(-?[0-9]+(\.[0-9]+)?)\s*" r",\s*(-?[0-9]+(\.[0-9]+)?)\s*\)"
    )

    def __init__(self, x, y, relative=False):
        """
        Args
        ----
        x: float or int
            X coordinate
        y: float or int
            Y coordinate
        relative: bool
            Coordinate is relative or absolute
        """
        self._x = float(x)
        self._y = float(y)
        self.relative = relative

    def __repr__(self):
        if self.relative:
            ret_str = "++"
        else:
            ret_str = ""
        return ret_str + "({},{})".format(self._x, self._y)

    def dumps(self):
        """Return representation."""

        return self.__repr__()

    @classmethod
    def from_str(cls, coordinate):
        """Build a TikZCoordinate object from a string."""

        m = cls._coordinate_str_regex.match(coordinate)

        if m is None:
            raise ValueError("invalid coordinate string")

        if m.group(1) == "++":
            relative = True
        else:
            relative = False

        return TikZCoordinate(float(m.group(2)), float(m.group(4)), relative=relative)

    def __eq__(self, other):
        if isinstance(other, tuple):
            # if comparing to a tuple, assume it to be an absolute coordinate.
            other_relative = False
            other_x = float(other[0])
            other_y = float(other[1])
        elif isinstance(other, TikZCoordinate):
            other_relative = other.relative
            other_x = other._x
            other_y = other._y
        else:
            raise TypeError("can only compare tuple and TiKZCoordinate types")

        # prevent comparison between relative and non relative
        # by returning False
        if other_relative != self.relative:
            return False

        # return comparison result
        return other_x == self._x and other_y == self._y

    def _arith_check(self, other):
        if isinstance(other, tuple):
            other_coord = TikZCoordinate(*other)
        elif isinstance(other, TikZCoordinate):
            if other.relative is True or self.relative is True:
                raise ValueError("refusing to add relative coordinates")
            other_coord = other
        else:
            raise TypeError("can only add tuple or TiKZCoordinate types")

        return other_coord

    def __add__(self, other):
        other_coord = self._arith_check(other)
        return TikZCoordinate(self._x + other_coord._x, self._y + other_coord._y)

    def __radd__(self, other):
        self.__add__(other)

    def __sub__(self, other):
        other_coord = self._arith_check(other)
        return TikZCoordinate(self._x - other_coord._y, self._y - other_coord._y)

    def distance_to(self, other):
        """Euclidean distance between two coordinates."""

        other_coord = self._arith_check(other)
        return math.sqrt(
            math.pow(self._x - other_coord._x, 2)
            + math.pow(self._y - other_coord._y, 2)
        )


class TikZObject(Container):
    """Abstract Class that most TikZ Objects inherits from."""

    def __init__(self, options=None):
        """
        Args
        ----
        options: list
            Options pertaining to the object
        """

        super(TikZObject, self).__init__()
        self.options = options


class TikZNodeAnchor(LatexObject):
    """Representation of a node's anchor point."""

    def __init__(self, node_handle, anchor_name):
        """
        Args
        ----
        node_handle: str
            Node's identifier
        anchor_name: str
            Name of the anchor
        """

        self.handle = node_handle
        self.anchor = anchor_name

    def __repr__(self):
        return "({}.{})".format(self.handle, self.anchor)

    def dumps(self):
        """Return a representation. Alias for consistency."""

        return self.__repr__()


class TikZNode(TikZObject):
    """A class that represents a TiKZ node."""

    _possible_anchors = ["north", "south", "east", "west"]

    def __init__(self, handle=None, options=None, at=None, text=None):
        """
        Args
        ----
        handle: str
            Node identifier
        options: list
            List of options
        at: TikZCoordinate
            Coordinate where node is placed
        text: str
            Body text of the node
        """
        super(TikZNode, self).__init__(options=options)

        self.handle = handle

        if isinstance(at, (TikZCoordinate, type(None))):
            self._node_position = at
        else:
            raise TypeError(
                "at parameter must be an object of the" "TikzCoordinate class"
            )

        self._node_text = text

    def dumps(self):
        """Return string representation of the node."""

        ret_str = []
        ret_str.append(Command("node", options=self.options).dumps())

        if self.handle is not None:
            ret_str.append("({})".format(self.handle))

        if self._node_position is not None:
            ret_str.append("at {}".format(str(self._node_position)))

        if self._node_text is not None:
            ret_str.append("{{{text}}};".format(text=self._node_text))
        else:
            ret_str.append("{};")

        return " ".join(ret_str)

    def get_anchor_point(self, anchor_name):
        """Return an anchor point of the node, if it exists."""

        if anchor_name in self._possible_anchors:
            return TikZNodeAnchor(self.handle, anchor_name)
        else:
            try:
                anchor = int(anchor_name.split("_")[1])
            except:
                anchor = None

            if anchor is not None:
                return TikZNodeAnchor(self.handle, str(anchor))

        raise ValueError('Invalid anchor name: "{}"'.format(anchor_name))

    def __getattr__(self, attr_name):
        try:
            point = self.get_anchor_point(attr_name)
            return point
        except ValueError:
            pass

        # raise AttributeError(
        #    'Invalid attribute requested: "{}"'.format(attr_name))


class TikZUserPath(LatexObject):
    """Represents a possible TikZ path."""

    def __init__(self, path_type, options=None):
        """
        Args
        ----
        path_type: str
            Type of path used
        options: Options
            List of options to add
        """
        super(TikZUserPath, self).__init__()
        self.path_type = path_type
        self.options = options

    def dumps(self):
        """Return path command representation."""

        ret_str = self.path_type

        if self.options is not None:
            ret_str += self.options.dumps()

        return ret_str


class TikZPathList(LatexObject):
    """Represents a path drawing."""

    _legal_path_types = ["--", "-|", "|-", "to", "rectangle", "circle", "arc", "edge"]

    def __init__(self, *args):
        """
        Args
        ----
        args: list
            A list of path elements
        """
        self._last_item_type = None
        self._arg_list = []

        # parse list and verify legality
        self._parse_arg_list(args)

    def append(self, item):
        """Add a new element to the current path."""
        self._parse_next_item(item)

    def _parse_next_item(self, item):
        # assume first item is a point
        if self._last_item_type is None:
            try:
                self._add_point(item)
            except (TypeError, ValueError):
                # not a point, do something
                raise TypeError(
                    "First element of path list must be a node identifier"
                    " or coordinate"
                )
        elif self._last_item_type == "point":
            # point after point is permitted, doesnt draw
            try:
                self._add_point(item)
                return
            except (ValueError, TypeError):
                # not a point, try path
                pass

            # will raise typeerror if wrong
            self._add_path(item)
        elif self._last_item_type == "path":
            # only point allowed after path
            original_exception = None
            try:
                self._add_point(item)
                return
            except (TypeError, ValueError) as ex:
                # check if trying to insert path after path
                try:
                    self._add_path(item, parse_only=True)
                    not_a_path = False
                    original_exception = ex
                except (TypeError, ValueError) as ex:
                    # not a path either!
                    not_a_path = True
                    original_exception = ex

            # disentangle exceptions
            if not_a_path is False:
                raise ValueError(
                    "only a point descriptor can come" " after a path descriptor"
                )

            if original_exception is not None:
                raise original_exception

    def _parse_arg_list(self, args):
        for item in args:
            self._parse_next_item(item)

    def _add_path(self, path, parse_only=False):
        if isinstance(path, str):
            if path in self._legal_path_types:
                _path = TikZUserPath(path)
            else:
                raise ValueError('Illegal user path type: "{}"'.format(path))
        elif isinstance(path, TikZUserPath):
            _path = path
        else:
            raise TypeError("Only string or TikZUserPath types are allowed")

        # add
        if parse_only is False:
            self._arg_list.append(_path)
            self._last_item_type = "path"
        else:
            return _path

    def _add_point(self, point, parse_only=False):
        if isinstance(point, str):
            try:
                _item = TikZCoordinate.from_str(point)
            except ValueError:
                raise ValueError('Illegal point string: "{}"'.format(point))
        elif isinstance(point, TikZCoordinate):
            _item = point
        elif isinstance(point, tuple):
            _item = TikZCoordinate(*point)
        elif isinstance(point, TikZNode):
            _item = "({})".format(point.handle)
        elif isinstance(point, TikZNodeAnchor):
            _item = point.dumps()
        else:
            raise TypeError(
                "Only str, tuple, TikZCoordinate,"
                "TikZNode or TikZNodeAnchor types are allowed,"
                " got: {}".format(type(point))
            )
        # add, finally
        if parse_only is False:
            self._arg_list.append(_item)
            self._last_item_type = "point"
        else:
            return _item

    def dumps(self):
        """Return representation of the path command."""

        ret_str = []
        for item in self._arg_list:
            if isinstance(item, TikZUserPath):
                ret_str.append(item.dumps())
            elif isinstance(item, TikZCoordinate):
                ret_str.append(item.dumps())
            elif isinstance(item, str):
                ret_str.append(item)

        return " ".join(ret_str)


class TikZPath(TikZObject):
    r"""The TikZ \path command."""

    def __init__(self, path=None, options=None):
        """
        Args
        ----
        path: TikZPathList
            A list of the nodes, path types in the path
        options: TikZOptions
            A list of options for the command
        """

        super(TikZPath, self).__init__(options=options)

        if isinstance(path, TikZPathList):
            self.path = path
        elif isinstance(path, list):
            self.path = TikZPathList(*path)
        elif path is None:
            self.path = TikZPathList()
        else:
            raise TypeError('argument "path" can only be of types list or TikZPathList')

    def append(self, element):
        """Append a path element to the current list."""
        self.path.append(element)

    def dumps(self):
        """Return a representation for the command."""

        ret_str = [Command("path", options=self.options).dumps()]

        ret_str.append(self.path.dumps())

        return " ".join(ret_str) + ";"


class TikZDraw(TikZPath):
    """A draw command is just a path command with the draw option."""

    def __init__(self, path=None, options=None):
        """
        Args
        ----
        path: TikZPathList
            A list of the nodes, path types in the path
        options: TikZOptions
            A list of options for the command
        """
        super(TikZDraw, self).__init__(path=path, options=options)

        # append option
        if self.options is not None:
            self.options.append_positional("draw")
        else:
            self.options = TikZOptions("draw")


class Plot(LatexObject):
    """A class representing a PGFPlot."""

    packages = [Package("pgfplots"), Command("pgfplotsset", "compat=newest")]

    def __init__(
        self, name=None, func=None, coordinates=None, error_bar=None, options=None
    ):
        """
        Args
        ----
        name: str
            Name of the plot.
        func: str
            A function that should be plotted.
        coordinates: list
            A list of exact coordinates tat should be plotted.

        options: str, list or `~.Options`
        """

        self.name = name
        self.func = func
        self.coordinates = coordinates
        self.error_bar = error_bar
        self.options = options

        super().__init__()

    def dumps(self):
        """Represent the plot as a string in LaTeX syntax.

        Returns
        -------
        str
        """

        string = Command("addplot", options=self.options).dumps()

        if self.coordinates is not None:
            string += " coordinates {%\n"

            if self.error_bar is None:
                for x, y in self.coordinates:
                    # ie: "(x,y)"
                    string += "(" + str(x) + "," + str(y) + ")%\n"

            else:
                for (x, y), (e_x, e_y) in zip(self.coordinates, self.error_bar):
                    # ie: "(x,y) +- (e_x,e_y)"
                    string += (
                        "("
                        + str(x)
                        + ","
                        + str(y)
                        + ") +- ("
                        + str(e_x)
                        + ","
                        + str(e_y)
                        + ")%\n"
                    )

            string += "};%\n%\n"

        elif self.func is not None:
            string += "{" + self.func + "};%\n%\n"

        if self.name is not None:
            string += Command("addlegendentry", self.name).dumps()

        super().dumps()

        return string



#!/usr/bin/python
"""
This example demonstrates several features of PyLaTeX.

It includes plain equations, tables, equations using numpy objects, tikz plots,
and figures.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import os

import numpy as np

from pylatex import (
    Alignat,
    Axis,
    Document,
    Figure,
    Math,
    Matrix,
    Plot,
    Section,
    Subsection,
    Tabular,
    TikZ,
)
from pylatex.utils import italic

if __name__ == "__main__":
    image_filename = os.path.join(os.path.dirname(__file__), "kitten.jpg")

    geometry_options = {"tmargin": "1cm", "lmargin": "10cm"}
    doc = Document(geometry_options=geometry_options)

    with doc.create(Section("The simple stuff")):
        doc.append("Some regular text and some")
        doc.append(italic("italic text. "))
        doc.append("\nAlso some crazy characters: $&#{}")
        with doc.create(Subsection("Math that is incorrect")):
            doc.append(Math(data=["2*3", "=", 9]))

        with doc.create(Subsection("Table of something")):
            with doc.create(Tabular("rc|cl")) as table:
                table.add_hline()
                table.add_row((1, 2, 3, 4))
                table.add_hline(1, 2)
                table.add_empty_row()
                table.add_row((4, 5, 6, 7))

    a = np.array([[100, 10, 20]]).T
    M = np.matrix([[2, 3, 4], [0, 0, 1], [0, 0, 2]])

    with doc.create(Section("The fancy stuff")):
        with doc.create(Subsection("Correct matrix equations")):
            doc.append(Math(data=[Matrix(M), Matrix(a), "=", Matrix(M * a)]))

        with doc.create(Subsection("Alignat math environment")):
            with doc.create(Alignat(numbering=False, escape=False)) as agn:
                agn.append(r"\frac{a}{b} &= 0 \\")
                agn.extend([Matrix(M), Matrix(a), "&=", Matrix(M * a)])

        with doc.create(Subsection("Beautiful graphs")):
            with doc.create(TikZ()):
                plot_options = "height=4cm, width=6cm, grid=major"
                with doc.create(Axis(options=plot_options)) as plot:
                    plot.append(Plot(name="model", func="-x^5 - 242"))

                    coordinates = [
                        (-4.77778, 2027.60977),
                        (-3.55556, 347.84069),
                        (-2.33333, 22.58953),
                        (-1.11111, -493.50066),
                        (0.11111, 46.66082),
                        (1.33333, -205.56286),
                        (2.55556, -341.40638),
                        (3.77778, -1169.24780),
                        (5.00000, -3269.56775),
                    ]

                    plot.append(Plot(name="estimate", coordinates=coordinates))

        with doc.create(Subsection("Cute kitten pictures")):
            with doc.create(Figure(position="h!")) as kitten_pic:
                kitten_pic.add_image(image_filename, width="120px")
                kitten_pic.add_caption("Look it's on its back")

    doc.generate_pdf("full", clean_tex=False)



# -*- coding: utf-8 -*-
"""
This module implements the class that deals with the full document.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import errno
import os
import subprocess
import sys

import pylatex.config as cf

from .base_classes import (
    Command,
    Container,
    Environment,
    LatexObject,
    SpecialArguments,
    UnsafeCommand,
)
from .errors import CompilerError
from .package import Package
from .utils import NoEscape, dumps_list, rm_temp_dir


class Document(Environment):
    r"""
    A class that contains a full LaTeX document.

    If needed, you can append stuff to the preamble or the packages.
    For instance, if you need to use ``\maketitle`` you can add the title,
    author and date commands to the preamble to make it work.

    """

    def __init__(
        self,
        default_filepath="default_filepath",
        *,
        documentclass="article",
        document_options=None,
        fontenc="T1",
        inputenc="utf8",
        font_size="normalsize",
        lmodern=True,
        textcomp=True,
        microtype=None,
        page_numbers=True,
        indent=None,
        geometry_options=None,
        data=None
    ):
        r"""
        Args
        ----
        default_filepath: str
            The default path to save files.
        documentclass: str or `~.Command`
            The LaTeX class of the document.
        document_options: str or `list`
            The options to supply to the documentclass
        fontenc: str
            The option for the fontenc package. If it is `None`, the fontenc
            package will not be loaded at all.
        inputenc: str
            The option for the inputenc package. If it is `None`, the inputenc
            package will not be loaded at all.
        font_size: str
            The font size to declare as normalsize
        lmodern: bool
            Use the Latin Modern font. This is a font that contains more glyphs
            than the standard LaTeX font.
        textcomp: bool
            Adds even more glyphs, for instance the Euro (€) sign.
        page_numbers: bool
            Adds the ability to add the last page to the document.
        indent: bool
            Determines whether or not the document requires indentation. If it
            is `None` it will use the value from the active config. Which is
            `True` by default.
        geometry_options: dict
            The options to supply to the geometry package
        data: list
            Initial content of the document.
        """

        self.default_filepath = default_filepath

        if isinstance(documentclass, Command):
            self.documentclass = documentclass
        else:
            self.documentclass = Command(
                "documentclass", arguments=documentclass, options=document_options
            )
        if indent is None:
            indent = cf.active.indent
        if microtype is None:
            microtype = cf.active.microtype

        # These variables are used by the __repr__ method
        self._fontenc = fontenc
        self._inputenc = inputenc
        self._lmodern = lmodern
        self._indent = indent
        self._microtype = microtype

        packages = []

        if fontenc is not None:
            packages.append(Package("fontenc", options=fontenc))
        if inputenc is not None:
            packages.append(Package("inputenc", options=inputenc))
        if lmodern:
            packages.append(Package("lmodern"))
        if textcomp:
            packages.append(Package("textcomp"))
        if page_numbers:
            packages.append(Package("lastpage"))
        if not indent:
            packages.append(Package("parskip"))
        if microtype:
            packages.append(Package("microtype"))

        if geometry_options is not None:
            packages.append(Package("geometry"))
            # Make sure we don't add this options command for an empty list,
            # because that breaks.
            if geometry_options:
                packages.append(
                    Command(
                        "geometry",
                        arguments=SpecialArguments(geometry_options),
                    )
                )

        super().__init__(data=data)

        # Usually the name is the class name, but if we create our own
        # document class, \begin{document} gets messed up.
        self._latex_name = "document"

        self.packages |= packages
        self.variables = []

        self.preamble = []

        if not page_numbers:
            self.change_document_style("empty")

        # No colors have been added to the document yet
        self.color = False
        self.meta_data = False

        self.append(Command(command=font_size))

    def _propagate_packages(self):
        r"""Propogate packages.

        Make sure that all the packages included in the previous containers
        are part of the full list of packages.
        """

        super()._propagate_packages()

        for item in self.preamble:
            if isinstance(item, LatexObject):
                if isinstance(item, Container):
                    item._propagate_packages()
                for p in item.packages:
                    self.packages.add(p)

    def dumps(self):
        """Represent the document as a string in LaTeX syntax.

        Returns
        -------
        str
        """

        head = self.documentclass.dumps() + "%\n"
        head += self.dumps_packages() + "%\n"
        head += dumps_list(self.variables) + "%\n"
        head += dumps_list(self.preamble) + "%\n"

        return head + "%\n" + super().dumps()

    def generate_tex(self, filepath=None):
        """Generate a .tex file for the document.

        Args
        ----
        filepath: str
            The name of the file (without .tex), if this is not supplied the
            default filepath attribute is used as the path.
        """

        super().generate_tex(self._select_filepath(filepath))

    def generate_pdf(
        self,
        filepath=None,
        *,
        clean=True,
        clean_tex=True,
        compiler=None,
        compiler_args=None,
        silent=True
    ):
        """Generate a pdf file from the document.

        Args
        ----
        filepath: str
            The name of the file (without .pdf), if it is `None` the
            ``default_filepath`` attribute will be used.
        clean: bool
            Whether non-pdf files created that are created during compilation
            should be removed.
        clean_tex: bool
            Also remove the generated tex file.
        compiler: `str` or `None`
            The name of the LaTeX compiler to use. If it is None, PyLaTeX will
            choose a fitting one on its own. Starting with ``latexmk`` and then
            ``pdflatex``.
        compiler_args: `list` or `None`
            Extra arguments that should be passed to the LaTeX compiler. If
            this is None it defaults to an empty list.
        silent: bool
            Whether to hide compiler output
        """

        if compiler_args is None:
            compiler_args = []

        # In case of newer python with the use of the cwd parameter
        # one can avoid to physically change the directory
        # to the destination folder
        python_cwd_available = sys.version_info >= (3, 6)

        filepath = self._select_filepath(filepath)
        if not os.path.basename(filepath):
            filepath = os.path.join(os.path.abspath(filepath), "default_basename")
        else:
            filepath = os.path.abspath(filepath)

        cur_dir = os.getcwd()
        dest_dir = os.path.dirname(filepath)

        if not python_cwd_available:
            os.chdir(dest_dir)

        self.generate_tex(filepath)

        if compiler is not None:
            compilers = ((compiler, []),)
        else:
            latexmk_args = ["--pdf"]

            compilers = (("latexmk", latexmk_args), ("pdflatex", []))

        main_arguments = ["--interaction=nonstopmode", filepath + ".tex"]

        check_output_kwargs = {}
        if python_cwd_available:
            check_output_kwargs = {"cwd": dest_dir}

        os_error = None

        for compiler, arguments in compilers:
            command = [compiler] + arguments + compiler_args + main_arguments

            try:
                output = subprocess.check_output(
                    command, stderr=subprocess.STDOUT, **check_output_kwargs
                )
            except (OSError, IOError) as e:
                # Use FileNotFoundError when python 2 is dropped
                os_error = e

                if os_error.errno == errno.ENOENT:
                    # If compiler does not exist, try next in the list
                    continue
                raise
            except subprocess.CalledProcessError as e:
                # For all other errors print the output and raise the error
                print(e.output.decode())
                raise
            else:
                if not silent:
                    print(output.decode())

            if clean:
                try:
                    # Try latexmk cleaning first
                    subprocess.check_output(
                        ["latexmk", "-c", filepath],
                        stderr=subprocess.STDOUT,
                        **check_output_kwargs
                    )
                except (OSError, IOError, subprocess.CalledProcessError):
                    # Otherwise just remove some file extensions.
                    extensions = ["aux", "log", "out", "fls", "fdb_latexmk"]

                    for ext in extensions:
                        try:
                            os.remove(filepath + "." + ext)
                        except (OSError, IOError) as e:
                            # Use FileNotFoundError when python 2 is dropped
                            if e.errno != errno.ENOENT:
                                raise
                rm_temp_dir()

            if clean_tex:
                os.remove(filepath + ".tex")  # Remove generated tex file

            # Compilation has finished, so no further compilers have to be
            # tried
            break

        else:
            # Notify user that none of the compilers worked.
            raise (
                CompilerError(
                    "No LaTex compiler was found\n"
                    "Either specify a LaTex compiler "
                    "or make sure you have latexmk or pdfLaTex installed."
                )
            )

        if not python_cwd_available:
            os.chdir(cur_dir)

    def _select_filepath(self, filepath):
        """Make a choice between ``filepath`` and ``self.default_filepath``.

        Args
        ----
        filepath: str
            the filepath to be compared with ``self.default_filepath``

        Returns
        -------
        str
            The selected filepath
        """

        if filepath is None:
            return self.default_filepath
        else:
            if os.path.basename(filepath) == "":
                filepath = os.path.join(
                    filepath, os.path.basename(self.default_filepath)
                )
            return filepath

    def change_page_style(self, style):
        r"""Alternate page styles of the current page.

        Args
        ----
        style: str
            value to set for the page style of the current page
        """

        self.append(Command("thispagestyle", arguments=style))

    def change_document_style(self, style):
        r"""Alternate page style for the entire document.

        Args
        ----
        style: str
            value to set for the document style
        """

        self.append(Command("pagestyle", arguments=style))

    def add_color(self, name, model, description):
        r"""Add a color that can be used throughout the document.

        Args
        ----
        name: str
            Name to set for the color
        model: str
            The color model to use when defining the color
        description: str
            The values to use to define the color
        """

        if self.color is False:
            self.packages.append(Package("color"))
            self.color = True

        self.preamble.append(
            Command("definecolor", arguments=[name, model, description])
        )

    def change_length(self, parameter, value):
        r"""Change the length of a certain parameter to a certain value.

        Args
        ----
        parameter: str
            The name of the parameter to change the length for
        value: str
            The value to set the parameter to
        """

        self.preamble.append(UnsafeCommand("setlength", arguments=[parameter, value]))

    def set_variable(self, name, value):
        r"""Add a variable which can be used inside the document.

        Variables are defined before the preamble. If a variable with that name
        has already been set, the new value will override it for future uses.
        This is done by appending ``\renewcommand`` to the document.

        Args
        ----
        name: str
            The name to set for the variable
        value: str
            The value to set for the variable
        """

        name_arg = "\\" + name
        variable_exists = False

        for variable in self.variables:
            if name_arg == variable.arguments._positional_args[0]:
                variable_exists = True
                break

        if variable_exists:
            renew = Command(
                command="renewcommand", arguments=[NoEscape(name_arg), value]
            )
            self.append(renew)
        else:
            new = Command(command="newcommand", arguments=[NoEscape(name_arg), value])
            self.variables.append(new)



# -*- coding: utf-8 -*-
r"""A test to make sure the document compiles with lmodern set to `False`."""

from pylatex import Document

doc = Document("no_lmodern", lmodern=False)
doc.append("test text")

doc.generate_pdf(clean=True, clean_tex=False, silent=False)



#!/usr/bin/python
"""
This example shows quantities functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
import quantities as pq

from pylatex import Document, Math, Quantity, Section, Subsection

if __name__ == "__main__":
    doc = Document()
    section = Section("Quantity tests")

    subsection = Subsection("Scalars with units")
    G = pq.constants.Newtonian_constant_of_gravitation
    moon_earth_distance = 384400 * pq.km
    moon_mass = 7.34767309e22 * pq.kg
    earth_mass = 5.972e24 * pq.kg
    moon_earth_force = G * moon_mass * earth_mass / moon_earth_distance**2
    q1 = Quantity(
        moon_earth_force.rescale(pq.newton),
        options={"round-precision": 4, "round-mode": "figures"},
    )
    math = Math(data=["F=", q1])
    subsection.append(math)
    section.append(subsection)

    subsection = Subsection("Scalars without units")
    world_population = 7400219037
    N = Quantity(
        world_population,
        options={"round-precision": 2, "round-mode": "figures"},
        format_cb="{0:23.17e}".format,
    )
    subsection.append(Math(data=["N=", N]))
    section.append(subsection)

    subsection = Subsection("Scalars with uncertainties")
    width = pq.UncertainQuantity(7.0, pq.meter, 0.4)
    length = pq.UncertainQuantity(6.0, pq.meter, 0.3)
    area = Quantity(
        width * length,
        options="separate-uncertainty",
        format_cb=lambda x: "{0:.1f}".format(float(x)),
    )
    subsection.append(Math(data=["A=", area]))
    section.append(subsection)

    doc.append(section)
    doc.generate_pdf("quantities_ex", clean_tex=False)



#!/usr/bin/python
"""
This example shows the functionality of the MiniPage element.

It creates a sample page filled with labels using the MiniPage element.

..  :copyright: (c) 2016 by Vladimir Gorovikov
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, HFill, LongTabu
from pylatex.utils import bold


def genenerate_longtabu():
    geometry_options = {
        "landscape": True,
        "margin": "0.5in",
        "headheight": "20pt",
        "headsep": "10pt",
        "includeheadfoot": True,
    }
    doc = Document(page_numbers=True, geometry_options=geometry_options)

    # Generate data table
    with doc.create(LongTabu("X[r] X[r] X[r] X[r] X[r] X[r]")) as data_table:
        header_row1 = ["Prov", "Num", "CurBal", "IntPay", "Total", "IntR"]
        data_table.add_row(header_row1, mapper=[bold])
        data_table.add_hline()
        data_table.add_empty_row()
        data_table.end_table_header()
        data_table.add_row(["Prov", "Num", "CurBal", "IntPay", "Total", "IntR"])
        row = ["PA", "9", "$100", "%10", "$1000", "Test"]
        for i in range(50):
            data_table.add_row(row)

    doc.append(bold("Grand Total:"))
    doc.append(HFill())
    doc.append(bold("Total"))

    doc.generate_pdf("longtabu", clean_tex=False)


genenerate_longtabu()



#!/usr/bin/python
"""
This example shows the functionality of the TextBlock element.

It creates a sample cheque to demonstrate the positioning of the elements on
the page.

..  :copyright: (c) 2016 by Vladimir Gorovikov
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import (
    Document,
    HorizontalSpace,
    HugeText,
    MediumText,
    MiniPage,
    SmallText,
    TextBlock,
    VerticalSpace,
)
from pylatex.utils import bold

geometry_options = {"margin": "0.5in"}
doc = Document(indent=False, geometry_options=geometry_options)
doc.change_length("\TPHorizModule", "1mm")
doc.change_length("\TPVertModule", "1mm")

with doc.create(MiniPage(width=r"\textwidth")) as page:
    with page.create(TextBlock(100, 0, 0)):
        page.append("**** Ten Thousand Dollars")

    with page.create(TextBlock(100, 0, 30)):
        page.append("COMPANY NAME")
        page.append("\nSTREET, ADDRESS")
        page.append("\nCITY, POSTAL CODE")

    with page.create(TextBlock(100, 150, 40)):
        page.append(HugeText(bold("VOID")))

    with page.create(TextBlock(80, 150, 0)):
        page.append("DATE")
        page.append(MediumText(bold("2016 06 07\n")))
        page.append(HorizontalSpace("10mm"))
        page.append(SmallText("Y/A M/M D/J"))

    with page.create(TextBlock(70, 150, 30)):
        page.append(MediumText(bold("$***** 10,000.00")))

    page.append(VerticalSpace("100mm"))

doc.generate_pdf("textblock", clean_tex=False)



# -*- coding: utf-8 -*-
"""
This module implements the class that deals with tables.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import re
from collections import Counter

import pylatex.config as cf

from .base_classes import (
    Command,
    Container,
    Environment,
    Float,
    LatexObject,
    UnsafeCommand,
)
from .errors import TableError, TableRowSizeError
from .package import Package
from .utils import NoEscape, _is_iterable, dumps_list

# The letters used to count the table width
COLUMN_LETTERS = {"l", "c", "r", "p", "m", "b", "X"}


def _get_table_width(table_spec):
    """Calculate the width of a table based on its spec.

    Args
    ----
    table_spec: str
        The LaTeX column specification for a table.


    Returns
    -------
    int
        The width of a table which uses the specification supplied.
    """

    # Remove things like {\bfseries}
    cleaner_spec = re.sub(r"{[^}]*}", "", table_spec)

    # Remove X[] in tabu environments so they dont interfere with column count
    cleaner_spec = re.sub(r"X\[(.*?(.))\]", r"\2", cleaner_spec)
    spec_counter = Counter(cleaner_spec)

    return sum(spec_counter[l] for l in COLUMN_LETTERS)


class Tabular(Environment):
    """A class that represents a tabular."""

    _repr_attributes_mapping = {
        "table_spec": "arguments",
        "pos": "options",
    }

    def __init__(
        self,
        table_spec,
        data=None,
        pos=None,
        *,
        row_height=None,
        col_space=None,
        width=None,
        booktabs=None,
        **kwargs
    ):
        """
        Args
        ----
        table_spec: str
            A string that represents how many columns a table should have and
            if it should contain vertical lines and where.
        pos: list
        row_height: float
            Specifies the heights of the rows in relation to the default
            row height
        col_space: str
            Specifies the spacing between table columns
        booktabs: bool
            Enable or disable booktabs style tables. These tables generally
            look nicer than regular tables. If this is `None` it will use the
            value of the ``booktabs`` attribte from the `~.active`
            configuration. This attribute is `False` by default.
        width: int
            The amount of columns that the table has. If this is `None` it is
            calculated based on the ``table_spec``, but this is only works for
            simple specs. In cases where this calculation is wrong override the
            width using this argument.

        References
        ----------
        * https://en.wikibooks.org/wiki/LaTeX/Tables#The_tabular_environment
        """

        if width is None:
            self.width = _get_table_width(table_spec)
        else:
            self.width = width

        if booktabs is None:
            booktabs = cf.active.booktabs
        self.booktabs = booktabs

        if self.booktabs:
            self.packages.add(Package("booktabs"))
            table_spec = "@{}%s@{}" % table_spec

        self.row_height = row_height if row_height is not None else cf.active.row_height
        self.col_space = col_space

        super().__init__(
            data=data, options=pos, arguments=NoEscape(table_spec), **kwargs
        )

        # Parameter that determines if the xcolor package has been added.
        self.color = False

    def dumps(self):
        r"""Turn the Latex Object into a string in Latex format."""

        string = ""

        if self.row_height is not None:
            row_height = Command(
                "renewcommand", arguments=[NoEscape(r"\arraystretch"), self.row_height]
            )
            string += row_height.dumps() + "%\n"

        if self.col_space is not None:
            col_space = Command(
                "setlength", arguments=[NoEscape(r"\tabcolsep"), self.col_space]
            )
            string += col_space.dumps() + "%\n"

        return string + super().dumps()

    def dumps_content(self, **kwargs):
        r"""Represent the content of the tabular in LaTeX syntax.

        This adds the top and bottomrule when using a booktabs style tabular.

        Args
        ----
        \*\*kwargs:
            Arguments that can be passed to `~.dumps_list`

        Returns
        -------
        string:
            A LaTeX string representing the
        """

        content = ""
        if self.booktabs:
            content += "\\toprule%\n"

        content += super().dumps_content(**kwargs)

        if self.booktabs:
            content += "\\bottomrule%\n"

        return NoEscape(content)

    def add_hline(self, start=None, end=None, *, color=None, cmidruleoption=None):
        r"""Add a horizontal line to the table.

        Args
        ----
        start: int
            At what cell the line should begin
        end: int
            At what cell the line should end
        color: str
            The hline color.
        cmidruleoption: str
            The option to be used for the booktabs cmidrule, i.e. the ``x`` in
            ``\cmidrule(x){1-3}``.
        """
        if self.booktabs:
            hline = "midrule"
            cline = "cmidrule"
            if cmidruleoption is not None:
                cline += "(" + cmidruleoption + ")"
        else:
            hline = "hline"
            cline = "cline"

        if color is not None:
            if not self.color:
                self.packages.append(Package("xcolor", options="table"))
                self.color = True
            color_command = Command(command="arrayrulecolor", arguments=color)
            self.append(color_command)

        if start is None and end is None:
            self.append(Command(hline))
        else:
            if start is None:
                start = 1
            elif end is None:
                end = self.width

            self.append(Command(cline, dumps_list([start, NoEscape("-"), end])))

    def add_empty_row(self):
        """Add an empty row to the table."""

        self.append(NoEscape((self.width - 1) * "&" + r"\\"))

    def add_row(self, *cells, color=None, escape=None, mapper=None, strict=True):
        """Add a row of cells to the table.

        Args
        ----
        cells: iterable, such as a `list` or `tuple`
            There's two ways to use this method. The first method is to pass
            the content of each cell as a separate argument. The second method
            is to pass a single argument that is an iterable that contains each
            contents.
        color: str
            The name of the color used to highlight the row
        mapper: callable or `list`
            A function or a list of functions that should be called on all
            entries of the list after converting them to a string,
            for instance bold
        strict: bool
            Check for correct count of cells in row or not.
        """

        if len(cells) == 1 and _is_iterable(cells):
            cells = cells[0]

        if escape is None:
            escape = self.escape

        # Propagate packages used in cells
        for c in cells:
            if isinstance(c, LatexObject):
                for p in c.packages:
                    self.packages.add(p)

        # Count cell contents
        cell_count = 0

        for c in cells:
            if isinstance(c, MultiColumn):
                cell_count += c.size
            else:
                cell_count += 1

        if strict and cell_count != self.width:
            msg = (
                "Number of cells added to table ({}) "
                "did not match table width ({})".format(cell_count, self.width)
            )
            raise TableRowSizeError(msg)

        if color is not None:
            if not self.color:
                self.packages.append(Package("xcolor", options="table"))
                self.color = True
            color_command = Command(command="rowcolor", arguments=color)
            self.append(color_command)

        self.append(
            dumps_list(cells, escape=escape, token="&", mapper=mapper) + NoEscape(r"\\")
        )


class Tabularx(Tabular):
    """A class that represents a tabularx environment."""

    packages = [Package("tabularx")]

    def __init__(self, *args, width_argument=NoEscape(r"\textwidth"), **kwargs):
        """
        Args
        ----
        width_argument:
            The width of the table. By default the table is as wide as the
            text.
        """
        super().__init__(*args, start_arguments=width_argument, **kwargs)


class MultiColumn(Container):
    """A class that represents a multicolumn inside of a table."""

    # TODO: Make this subclass of ContainerCommand

    def __init__(self, size, *, align="c", color=None, data=None):
        """
        Args
        ----
        size: int
            The amount of columns that this cell should fill.
        align: str
            How to align the content of the cell.
        data: str, list or `~.LatexObject`
            The content of the cell.
        color: str
            The color for the MultiColumn
        """

        self.size = size
        self.align = align

        super().__init__(data=data)

        # Add a cell color to the MultiColumn
        if color is not None:
            self.packages.append(Package("xcolor", options="table"))
            color_command = Command("cellcolor", arguments=color)
            self.append(color_command)

    def dumps(self):
        """Represent the multicolumn as a string in LaTeX syntax.

        Returns
        -------
        str
        """

        args = [self.size, self.align, self.dumps_content()]
        string = Command(self.latex_name, args).dumps()

        return string


class MultiRow(Container):
    """A class that represents a multirow in a table."""

    # TODO: Make this subclass CommandBase and Container

    packages = [Package("multirow")]

    def __init__(self, size, *, width="*", color=None, data=None):
        """
        Args
        ----
        size: int
            The amount of rows that this cell should fill.
        width: str
            Width of the cell. The default is ``*``, which means the content's
            natural width.
        data: str, list or `~.LatexObject`
            The content of the cell.
        color: str
            The color for the MultiRow
        """

        self.size = size
        self.width = width

        super().__init__(data=data)

        if color is not None:
            self.packages.append(Package("xcolor", options="table"))
            color_command = Command("cellcolor", arguments=color)
            self.append(color_command)

    def dumps(self):
        """Represent the multirow as a string in LaTeX syntax.

        Returns
        -------
        str
        """

        args = [self.size, self.width, self.dumps_content()]
        string = Command(self.latex_name, args).dumps()

        return string


class Table(Float):
    """A class that represents a table float."""


class Tabu(Tabular):
    """A class that represents a tabu (more flexible table)."""

    packages = [Package("tabu")]

    def __init__(
        self,
        table_spec,
        data=None,
        pos=None,
        *,
        row_height=None,
        col_space=None,
        width=None,
        booktabs=None,
        spread=None,
        to=None,
        **kwargs
    ):
        """
        Args
        ----
        table_spec: str
            A string that represents how many columns a table should have and
            if it should contain vertical lines and where.
        pos: list
        row_height: float
            Specifies the heights of the rows in relation to the default
            row height
        col_space: str
            Specifies the spacing between table columns
        booktabs: bool
            Enable or disable booktabs style tables. These tables generally
            look nicer than regular tables. If this is `None` it will use the
            value of the ``booktabs`` attribte from the `~.active`
            configuration. This attribute is `False` by default.
        spread: str
            Specifies the Tabu table should add a given amount of 'padding' to
            the width of the table. This should be a latex dimension;
            for example: "0 pt" or "1in"
        to: str
            Specifies the Tabu table should extend to a given width.
            This should be a latex dimension; for example '4in'
        width: int
            The amount of columns that the table has. If this is `None` it is
            calculated based on the ``table_spec``, but this is only works for
            simple specs. In cases where this calculation is wrong override the
            width using this argument.

        References
        ----------
        * https://en.wikibooks.org/wiki/LaTeX/Tables#The_tabular_environment
        """

        super().__init__(
            table_spec,
            data,
            pos,
            row_height=row_height,
            col_space=col_space,
            width=width,
            booktabs=booktabs,
            **kwargs
        )

        self._preamble = ""
        if spread:
            self._preamble = "spread " + spread
        elif to:
            self._preamble = "to " + to

    def dumps(self):
        """Turn the tabu object into a string in Latex format."""

        _s = super().dumps()

        # Tabu tables support a unusual syntax:
        # \begin{tabu} spread 0pt {<col format...>}
        #
        # Since this syntax isn't common, it doesn't make
        # sense to support it in the baseclass (e.g., Environment)
        # rather, here we fix the LaTeX string post-hoc
        if self._preamble:
            if _s.startswith(r"\begin{longtabu}"):
                _s = _s[:16] + self._preamble + _s[16:]
            elif _s.startswith(r"\begin{tabu}"):
                _s = _s[:12] + self._preamble + _s[12:]
            else:
                raise TableError(
                    "Can't apply preamble to Tabu table "
                    "(unexpected initial command sequence)"
                )

        return _s


class LongTable(Tabular):
    """A class that represents a longtable (multipage table)."""

    packages = [Package("longtable")]

    header = False
    foot = False
    lastFoot = False  # noqa, casing is needed for backwards compatibility

    def end_table_header(self):
        r"""End the table header which will appear on every page."""

        if self.header:
            msg = "Table already has a header"
            raise TableError(msg)

        self.header = True

        self.append(Command(r"endhead"))

    def end_table_footer(self):
        r"""End the table foot which will appear on every page."""

        if self.foot:
            msg = "Table already has a foot"
            raise TableError(msg)

        self.foot = True

        self.append(Command("endfoot"))

    def end_table_last_footer(self):
        r"""End the table foot which will appear on the last page."""

        if self.lastFoot:
            msg = "Table already has a last foot"
            raise TableError(msg)

        self.lastFoot = True

        self.append(Command("endlastfoot"))


class LongTabu(LongTable, Tabu):
    """A class that represents a longtabu (more flexible multipage table)."""


class LongTabularx(Tabularx, LongTable):
    """A class that represents a long version of the tabularx environment.

    This uses the ``ltablex`` package. This package modifies the ``tabularx``
    environment so that it can be spread over multiple pages. This has the
    sideeffect that using this class in a document spreads all `Tabularx`
    elements in that document over multiple pages as well.
    """

    _latex_name = "tabularx"

    packages = [Package("ltablex")]


class ColumnType(UnsafeCommand):
    r"""A class representing a new column type.

    It uses the ``\newcolumntype`` command, for a thorough explanation see
    `this StackExchange question <https://tex.stackexchange.com/
    questions/257128/how-does-the-newcolumntype-command-work>`_.
    """

    _repr_attributes_mapping = {"name": "arguments", "parameters": "options"}

    def __init__(self, name, base, modifications, *, parameters=None):
        """
        Args
        ----
        name: str
            The name of the new column type (a single letter)
        base: str
            The name of the column type that the new one is based on (a single
            letter)
        modifications: str
            The modifications to be made to the base column type
        parameters: int
            The number of # parameters inside the modifications string, if this
            is `None` this is calculated automatically.
        """

        # repr vars
        self._base = base
        self._modifications = modifications

        COLUMN_LETTERS.add(name)

        if parameters is None:
            # count the number of non escaped #<number> parameters
            parameters = len(re.findall(r"(?<!\\)#\d", modifications))
            parameters += len(re.findall(r"(?<!\\)#\d", base))

        if parameters == 0:
            parameters = None

        modified = r">{%s\arraybackslash}%s" % (modifications, base)

        super().__init__(
            command="newcolumntype",
            arguments=name,
            options=parameters,
            extra_arguments=modified,
        )



# -*- coding: utf-8 -*-
"""
This module implements the class that deals with packages.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import CommandBase


class Package(CommandBase):
    """A class that represents a package."""

    _latex_name = "usepackage"

    _repr_attributes_mapping = {
        "name": "arguments",
    }

    def __init__(self, name, options=None):
        """
        Args
        ----
        name: str
            Name of the package.
        options: `str`, `list` or `~.Options`
            Options of the package.

        """

        super().__init__(arguments=name, options=options)



# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with adding frames.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import ContainerCommand, Environment
from .package import Package


class MdFramed(Environment):
    """A class that defines an mdframed environment."""

    packages = [Package("mdframed")]


class FBox(ContainerCommand):
    """A class that defines an fbox ContainerCommand."""



#!/usr/bin/python
"""
How to represent your own LaTeX commands and environments in PyLaTeX.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, Section, UnsafeCommand
from pylatex.base_classes import Arguments, CommandBase, Environment
from pylatex.package import Package
from pylatex.utils import NoEscape


class ExampleEnvironment(Environment):
    """
    A class representing a custom LaTeX environment.

    This class represents a custom LaTeX environment named
    ``exampleEnvironment``.
    """

    _latex_name = "exampleEnvironment"
    packages = [Package("mdframed")]


class ExampleCommand(CommandBase):
    """
    A class representing a custom LaTeX command.

    This class represents a custom LaTeX command named
    ``exampleCommand``.
    """

    _latex_name = "exampleCommand"
    packages = [Package("color")]


# Create a new document
doc = Document()
with doc.create(Section("Custom commands")):
    doc.append(
        NoEscape(
            r"""
        The following is a demonstration of a custom \LaTeX{}
        command with a couple of parameters.
        """
        )
    )

    # Define the new command
    new_comm = UnsafeCommand(
        "newcommand",
        "\exampleCommand",
        options=3,
        extra_arguments=r"\color{#1} #2 #3 \color{black}",
    )
    doc.append(new_comm)

    # Use our newly created command with different arguments
    doc.append(ExampleCommand(arguments=Arguments("blue", "Hello", "World!")))
    doc.append(ExampleCommand(arguments=Arguments("green", "Hello", "World!")))
    doc.append(ExampleCommand(arguments=Arguments("red", "Hello", "World!")))

with doc.create(Section("Custom environments")):
    doc.append(
        NoEscape(
            r"""
        The following is a demonstration of a custom \LaTeX{}
        environment using the mdframed package.
        """
        )
    )

    # Define a style for our box
    mdf_style_definition = UnsafeCommand(
        "mdfdefinestyle",
        arguments=[
            "my_style",
            ("linecolor=#1," "linewidth=#2," "leftmargin=1cm," "leftmargin=1cm"),
        ],
    )

    # Define the new environment using the style definition above
    new_env = UnsafeCommand(
        "newenvironment",
        "exampleEnvironment",
        options=2,
        extra_arguments=[
            mdf_style_definition.dumps() + r"\begin{mdframed}[style=my_style]",
            r"\end{mdframed}",
        ],
    )
    doc.append(new_env)

    # Usage of the newly created environment
    with doc.create(ExampleEnvironment(arguments=Arguments("red", 3))) as environment:
        environment.append("This is the actual content")

# Generate pdf
doc.generate_pdf("own_commands_ex", clean_tex=False)



#!/usr/bin/env python

import os

from pylatex.utils import fix_filename


def test_no_dots():
    fname = "aaa"
    assert fix_filename(fname) == fname


def test_one_dot():
    fname = "aa.a"
    assert fix_filename(fname) == fname


def test_two_dots():
    fname = "aa.a.a"
    original_os_name = os.name
    try:
        os.name = "posix"
        assert fix_filename(fname) == "{aa.a}.a"
        os.name = "nt"
        assert fix_filename(fname) == "aa.a.a"
    finally:
        os.name = original_os_name


def test_three_dots():
    fname = "abc.def.fgh.ijk"
    assert fix_filename(fname) == "{abc.def.fgh}.ijk"


def test_path_and_three_dots():
    fname = "/auu/bcd/abc.def.fgh.ijk"
    assert fix_filename(fname) == "/auu/bcd/{abc.def.fgh}.ijk"


def test_dots_in_path_none_in_filename():
    fname = "/au.u/b.c.d/abc"
    assert fix_filename(fname) == "/au.u/b.c.d/abc"


def test_dots_in_path_one_in_filename():
    fname = "/au.u/b.c.d/abc.def"
    assert fix_filename(fname) == "/au.u/b.c.d/abc.def"


def test_dots_in_path_and_multiple_in_filename():
    fname = "/au.u/b.c.d/abc.def.fgh.ijk"
    assert fix_filename(fname) == "/au.u/b.c.d/{abc.def.fgh}.ijk"


def test_tilde_in_filename():
    fname = "/etc/local/foo.bar.baz/foo~1/document.pdf"
    assert (
        fix_filename(fname) == "\detokenize{/etc/local/foo.bar.baz/foo~1/document.pdf}"
    )



# -*- coding: utf-8 -*-
"""
This module implements the classes that deal with LaTeX lists.

These lists are specifically enumerate, itemize and description.

..  :copyright: (c) 2015 by Sean McLemon.
    :license: MIT, see License for more details.
"""

from pylatex.utils import NoEscape

from .base_classes import Command, Environment, Options
from .package import Package


class List(Environment):
    """A base class that represents a list."""

    #: List environments cause compile errors when they do not contain items.
    #: This is why they are omitted fully if they are empty.
    omit_if_empty = True

    def add_item(self, s):
        """Add an item to the list.

        Args
        ----
        s: str or `~.LatexObject`
            The item itself.
        """
        self.append(Command("item"))
        self.append(s)


class Enumerate(List):
    """A class that represents an enumerate list."""

    def __init__(self, enumeration_symbol=None, *, options=None, **kwargs):
        r"""
        Args
        ----
        enumeration_symbol: str
            The enumeration symbol to use, see the `enumitem
            <https://www.ctan.org/pkg/enumitem>`_ documentation to see what
            can be used here. This argument is not escaped as it usually
            should usually contain commands, so do not use user input here.
        options: str or list or `.Options`
            Custom options to be added to the enumerate list. These options are
            merged with the options created by ``enumeration_symbol``.
        """

        self._enumeration_symbol = enumeration_symbol

        if enumeration_symbol is not None:
            self.packages.add(Package("enumitem"))

            if options is not None:
                options = Options(options)
            else:
                options = Options()
            options._positional_args.append(NoEscape("label=" + enumeration_symbol))

        super().__init__(options=options, **kwargs)


class Itemize(List):
    """A class that represents an itemize list."""


class Description(List):
    """A class that represents a description list."""

    def add_item(self, label, s):
        """Add an item to the list.

        Args
        ----
        label: str
            Description of the item.
        s: str or `~.LatexObject`
            The item itself.
        """
        self.append(Command("item", options=label))
        self.append(s)



# -*- coding: utf-8 -*-
"""
This module implements the ability to use of different configurations.

The current active configuration is `pylatex.config.active`. This variable can
simply be changed to another configuration and that will be used.

It is also possible to use the `~.Version1.use` method to do this temporarily
in a specific context.

..  :copyright: (c) 2016 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from contextlib import contextmanager


class Version1:
    """The config used to get the behaviour of v1.x.y of the library.

    The default attributes are::

        indent = True
        booktabs = False
        microtype = False
        row_height = None
    """

    indent = True
    booktabs = False
    microtype = False
    row_height = None

    def __init__(self, **kwargs):
        """
        Args
        ----
        kwargs:
            Key value pairs of the default attributes that should be overridden
        """
        for k, v in kwargs.items():
            setattr(self, k, v)

    @contextmanager
    def use(self):
        """Use the config temporarily in specific context.

        A simple usage example::

            with Version1(indent=False).use():
                # Do stuff where indent should be False
                ...


        """
        global active
        prev = active
        active = self
        yield
        active = prev

    @contextmanager
    def change(self, **kwargs):
        """Override some attributes of the config in a specific context.

        A simple usage example::

            with pylatex.config.active.change(indent=False):
                # Do stuff where indent should be False
                ...

        Args
        ----
        kwargs:
            Key value pairs of the default attributes that should be overridden
        """

        old_attrs = {}

        for k, v in kwargs.items():
            old_attrs[k] = getattr(self, k, v)
            setattr(self, k, v)

        yield self  # allows with ... as ...

        for k, v in old_attrs.items():
            setattr(self, k, v)


#: The default configuration in this release. Currently the same as `Version1`
Default = Version1


class Version2(Version1):
    """The config used to get the behaviour of v2.x.y of the library.

    The default attributes are::

        indent = False
        booktabs = True
        microtype = True
        row_height = 1.3
    """

    indent = False
    booktabs = True
    microtype = True
    row_height = 1.3


#: The default configuration in the nxt major release. Currently the same as
#: `Version2`.
NextMajor = Version2

#: The current active configuration
active = Default()



#!/usr/bin/python


"""Test to validate that Environments uphold contract of base classes."""

from pylatex.base_classes import Environment


def test_alltt():
    class AllTT(Environment):
        escape = False
        content_separator = "\n"

    alltt = AllTT()
    alltt.append("This is alltt content\nIn two lines")
    s = alltt.dumps()
    assert s.startswith("\\begin{alltt}\nThis is"), "Unexpected start of environment"
    assert s.endswith("two lines\n\\end{alltt}"), "Unexpected end of environment"



# -*- coding: utf-8 -*-
"""
This module implements the base LaTeX object.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from abc import ABCMeta, abstractmethod
from inspect import getfullargspec
from reprlib import recursive_repr

from ordered_set import OrderedSet

from ..utils import dumps_list


class _CreatePackages(ABCMeta):
    def __init__(cls, name, bases, d):  # noqa
        packages = OrderedSet()

        for b in bases:
            if hasattr(b, "packages"):
                packages |= b.packages

        if "packages" in d:
            packages |= d["packages"]

        cls.packages = packages

        super().__init__(name, bases, d)


class LatexObject(metaclass=_CreatePackages):
    """The class that every other LaTeX class is a subclass of.

    This class implements the main methods that every LaTeX object needs. For
    conversion to LaTeX formatted strings it implements the dumps, dump and
    generate_tex methods. It also provides the methods that can be used to
    represent the packages required by the LatexObject.
    """

    _latex_name = None
    _star_latex_name = False  # latex_name + ('*' if True else '')

    #: Set this to an iterable to override the list of default repr
    #: attributes.
    _repr_attributes_override = None
    #: Set this to a dict to change some of the default repr attributes to
    #: other attributes. The key is the old one, the value the new one.
    _repr_attributes_mapping = None

    #: Set on a class to make instances default to a certain kind of escaping
    _default_escape = True

    #: Only set this directly by changing the cls.escape
    _escape = None

    @property
    def escape(self):
        """Determine whether or not to escape content of this class.

        This defaults to `True` for most classes.
        """
        if self._escape is not None:
            return self._escape
        if self._default_escape is not None:
            return self._default_escape
        return True

    @escape.setter
    def escape(self, value):
        """Escape flag setter - to be used at object level."""
        self._escape = value

    #: Start a new paragraph before this environment.
    begin_paragraph = False

    #: Start a new paragraph after this environment.
    end_paragraph = False

    #: Same as enabling `begin_paragraph` and `end_paragraph`, so
    #: effectively placing this element in its own paragraph.
    separate_paragraph = False

    def __init__(self):
        # TODO: only create a copy of packages when it will
        # Create a copy of the packages attribute, so changing it in an
        # instance will not change the class default.
        self.packages = self.packages.copy()

    @recursive_repr()
    def __repr__(self):
        """Create a printable representation of the object."""

        return (
            self.__class__.__name__
            + "("
            + ", ".join(map(repr, self._repr_values))
            + ")"
        )

    @property
    def _repr_values(self):
        """Return values that are to be shown in repr string."""

        def getattr_better(obj, field):
            try:
                return getattr(obj, field)
            except AttributeError as e:
                try:
                    return getattr(obj, "_" + field)
                except AttributeError:
                    raise e

        return (getattr_better(self, attr) for attr in self._repr_attributes)

    @property
    def _repr_attributes(self):
        """Return attributes that should be part of the repr string."""
        if self._repr_attributes_override is None:
            # Default to init arguments
            attrs = getfullargspec(self.__init__).args[1:]
            mapping = self._repr_attributes_mapping
            if mapping:
                attrs = [mapping[a] if a in mapping else a for a in attrs]
            return attrs

        return self._repr_attributes_override

    @property
    def latex_name(self):
        """Return the name of the class used in LaTeX.

        It can be `None` when the class doesn't have a name.
        """
        star = "*" if self._star_latex_name else ""
        if self._latex_name is not None:
            return self._latex_name + star
        return self.__class__.__name__.lower() + star

    @latex_name.setter
    def latex_name(self, value):
        self._latex_name = value

    @abstractmethod
    def dumps(self):
        """Represent the class as a string in LaTeX syntax.

        This method should be implemented by any class that subclasses this
        class.
        """

    def dump(self, file_w):
        """Write the LaTeX representation of the class to a file.

        Args
        ----
        file_w: io.TextIOBase
            The file object in which to save the data

        """

        file_w.write(self.dumps())

    def generate_tex(self, filepath):
        """Generate a .tex file.

        Args
        ----
        filepath: str
            The name of the file (without .tex)
        """

        with open(filepath + ".tex", "w", encoding="utf-8") as newf:
            self.dump(newf)

    def dumps_packages(self):
        """Represent the packages needed as a string in LaTeX syntax.

        Returns
        -------
        list
        """

        return dumps_list(self.packages)

    def dump_packages(self, file_w):
        """Write the LaTeX representation of the packages to a file.

        Args
        ----
        file_w: io.TextIOBase
            The file object in which to save the data

        """

        file_w.write(self.dumps_packages())

    def dumps_as_content(self):
        """Create a string representation of the object as content.

        This is currently only used to add new lines before and after the
        output of the dumps function. These can be added or removed by changing
        the `begin_paragraph`, `end_paragraph` and `separate_paragraph`
        attributes of the class.
        """

        string = self.dumps()

        if self.separate_paragraph or self.begin_paragraph:
            string = "\n\n" + string.lstrip("\n")

        if self.separate_paragraph or self.end_paragraph:
            string = string.rstrip("\n") + "\n\n"

        return string



# -*- coding: utf-8 -*-
"""
This module implements the class that deals with graphics.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

import posixpath
import uuid

from .base_classes import Float, UnsafeCommand
from .package import Package
from .utils import NoEscape, escape_latex, fix_filename, make_temp_dir


class Figure(Float):
    """A class that represents a Figure environment."""

    def add_image(
        self,
        filename,
        *,
        width=NoEscape(r"0.8\textwidth"),
        placement=NoEscape(r"\centering")
    ):
        """Add an image to the figure.

        Args
        ----
        filename: str
            Filename of the image.
        width: str
            The width of the image
        placement: str
            Placement of the figure, `None` is also accepted.

        """

        if width is not None:
            if self.escape:
                width = escape_latex(width)

            width = "width=" + str(width)

        if placement is not None:
            self.append(placement)

        self.append(
            StandAloneGraphic(image_options=width, filename=fix_filename(filename))
        )

    def _save_plot(self, *args, extension="pdf", **kwargs):
        """Save the plot.

        Returns
        -------
        str
            The basename with which the plot has been saved.
        """
        import matplotlib.pyplot as plt

        tmp_path = make_temp_dir()
        filename = "{}.{}".format(str(uuid.uuid4()), extension.strip("."))
        filepath = posixpath.join(tmp_path, filename)

        plt.savefig(filepath, *args, **kwargs)
        return filepath

    def add_plot(self, *args, extension="pdf", **kwargs):
        """Add the current Matplotlib plot to the figure.

        The plot that gets added is the one that would normally be shown when
        using ``plt.show()``.

        Args
        ----
        args:
            Arguments passed to plt.savefig for displaying the plot.
        extension : str
            extension of image file indicating figure file type
        kwargs:
            Keyword arguments passed to plt.savefig for displaying the plot. In
            case these contain ``width`` or ``placement``, they will be used
            for the same purpose as in the add_image command. Namely the width
            and placement of the generated plot in the LaTeX document.
        """

        add_image_kwargs = {}

        for key in ("width", "placement"):
            if key in kwargs:
                add_image_kwargs[key] = kwargs.pop(key)

        filename = self._save_plot(*args, extension=extension, **kwargs)

        self.add_image(filename, **add_image_kwargs)


class SubFigure(Figure):
    """A class that represents a subfigure from the subcaption package."""

    packages = [Package("subcaption")]

    #: By default a subfigure is not on its own paragraph since that looks
    #: weird inside another figure.
    separate_paragraph = False

    _repr_attributes_mapping = {
        "width": "arguments",
    }

    def __init__(self, width=NoEscape(r"0.45\linewidth"), **kwargs):
        """
        Args
        ----
        width: str
            Width of the subfigure itself. It needs a width because it is
            inside another figure.

        """

        super().__init__(arguments=width, **kwargs)

    def add_image(self, filename, *, width=NoEscape(r"\linewidth"), placement=None):
        """Add an image to the subfigure.

        Args
        ----
        filename: str
            Filename of the image.
        width: str
            Width of the image in LaTeX terms.
        placement: str
            Placement of the figure, `None` is also accepted.
        """

        super().add_image(filename, width=width, placement=placement)


class StandAloneGraphic(UnsafeCommand):
    r"""A class representing a stand alone image."""

    _latex_name = "includegraphics"

    packages = [Package("graphicx")]

    _repr_attributes_mapping = {"filename": "arguments", "image_options": "options"}

    def __init__(
        self,
        filename,
        image_options=NoEscape(r"width=0.8\textwidth"),
        extra_arguments=None,
    ):
        r"""
        Args
        ----
        filename: str
            The path to the image file
        image_options: str or `list`
            Specifies the options for the image (ie. height, width)
        """

        arguments = [NoEscape(filename)]

        super().__init__(
            command=self._latex_name,
            arguments=arguments,
            options=image_options,
            extra_arguments=extra_arguments,
        )



#!/usr/bin/python
"""
This example shows basic document generation functionality.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Command, Document, Section, Subsection
from pylatex.utils import NoEscape, italic


def fill_document(doc):
    """Add a section, a subsection and some text to the document.

    :param doc: the document
    :type doc: :class:`pylatex.document.Document` instance
    """
    with doc.create(Section("A section")):
        doc.append("Some regular text and some ")
        doc.append(italic("italic text. "))

        with doc.create(Subsection("A subsection")):
            doc.append("Also some crazy characters: $&#{}")


if __name__ == "__main__":
    # Basic document
    doc = Document("basic")
    fill_document(doc)

    doc.generate_pdf(clean_tex=False)
    doc.generate_tex()

    # Document with `\maketitle` command activated
    doc = Document()

    doc.preamble.append(Command("title", "Awesome Title"))
    doc.preamble.append(Command("author", "Anonymous author"))
    doc.preamble.append(Command("date", NoEscape(r"\today")))
    doc.append(NoEscape(r"\maketitle"))

    fill_document(doc)

    doc.generate_pdf("basic_maketitle", clean_tex=False)

    # Add stuff to the document
    with doc.create(Section("A second section")):
        doc.append("Some text.")

    doc.generate_pdf("basic_maketitle2", clean_tex=False)
    tex = doc.dumps()  # The document as string in LaTeX syntax



# -*- coding: utf-8 -*-
"""
This module implements the section type classes.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""


from .base_classes import Command, Container
from .labelref import Label, Marker


class Section(Container):
    """A class that represents a section."""

    #: A section should normally start in its own paragraph
    end_paragraph = True

    #: Default prefix to use with Marker
    marker_prefix = "sec"

    #: Number the sections when the section element is compatible,
    #: by changing the `~.Section` class default all
    #: subclasses will also have the new default.
    numbering = True

    def __init__(self, title, numbering=None, *, label=True, **kwargs):
        """
        Args
        ----
        title: str
            The section title.
        numbering: bool
            Add a number before the section title.
        label: Label or bool or str
            Can set a label manually or use a boolean to set
            preference between automatic or no label
        """

        self.title = title

        if numbering is not None:
            self.numbering = numbering
        if isinstance(label, Label):
            self.label = label
        elif isinstance(label, str):
            if ":" in label:
                label = label.split(":", 1)
                self.label = Label(Marker(label[1], label[0]))
            else:
                self.label = Label(Marker(label, self.marker_prefix))
        elif label:
            self.label = Label(Marker(title, self.marker_prefix))
        else:
            self.label = None

        super().__init__(**kwargs)

    def dumps(self):
        """Represent the section as a string in LaTeX syntax.

        Returns
        -------
        str

        """

        if not self.numbering:
            num = "*"
        else:
            num = ""

        string = Command(self.latex_name + num, self.title).dumps()
        if self.label is not None:
            string += "%\n" + self.label.dumps()
        string += "%\n" + self.dumps_content()

        return string


class Part(Section):
    """A class that represents a part."""

    marker_prefix = "part"


class Chapter(Section):
    """A class that represents a chapter."""

    marker_prefix = "chap"


class Subsection(Section):
    """A class that represents a subsection."""

    marker_prefix = "subsec"


class Subsubsection(Section):
    """A class that represents a subsubsection."""

    marker_prefix = "ssubsec"


class Paragraph(Section):
    """A class that represents a paragraph."""

    marker_prefix = "para"


class Subparagraph(Section):
    """A class that represents a subparagraph."""

    marker_prefix = "subpara"



#!/usr/bin/env python

from pylatex import Document, Section
from pylatex.utils import escape_latex


def test():
    doc = Document("utils_escape_latex")
    section = Section("Escape LaTeX characters test")

    text = escape_latex(
        """\
    & (ampersand)
    % (percent)
    $ (dollar)
    # (number)
    _ (underscore)
    { (left curly brace)
    } (right curly brace)
    ~ (tilde)
    ^ (caret)
    \\ (backslash)
    --- (three minuses)
    a\xA0a (non breaking space)
    [ (left bracket)
    ] (right bracket)
    """
    )

    section.append(text)
    doc.append(section)

    doc.generate_pdf()


if __name__ == "__main__":
    test()



#!/usr/bin/env python

from pylatex.base_classes import LatexObject
from pylatex.utils import _latex_item_to_string

TEST_STR = "hello"


def test_string():
    name = "abc"
    assert _latex_item_to_string(name) == name


def test_user_latex_object():
    class TestLatexObject(LatexObject):
        def dumps(self):
            return TEST_STR

    assert _latex_item_to_string(TestLatexObject()) == TEST_STR


def test_foreign_object():
    class ForeignObject:
        def dumps(self):
            return 15

        def __str__(self):
            return TEST_STR

    assert _latex_item_to_string(ForeignObject()) == TEST_STR



"""
Baseclasses that can be used to create classes representing LaTeX objects.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .command import (
    Arguments,
    Command,
    CommandBase,
    Options,
    SpecialArguments,
    SpecialOptions,
    UnsafeCommand,
)
from .containers import Container, ContainerCommand, Environment
from .float import Float
from .latex_object import LatexObject

# Old names of the base classes for backwards compatibility
BaseLaTeXClass = LatexObject
BaseLaTeXContainer = Container
BaseLaTeXNamedContainer = Environment



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# PyLaTeX documentation build configuration file, created by
# sphinx-quickstart on Thu Jun 18 15:35:21 2015.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# Needed for old sphinx version to work
import collections
import sys

if sys.version_info >= (3, 10):
    collections.Callable = collections.abc.Callable

import inspect
import os

import sphinx_rtd_theme

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.insert(0, os.path.abspath("../../"))
from pylatex import __version__

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.mathjax",
    "sphinx.ext.ifconfig",
    "sphinx.ext.intersphinx",
    "sphinx.ext.autosummary",
    "sphinx.ext.extlinks",
    "sphinx.ext.napoleon",
    "sphinx.ext.linkcode",
]

napoleon_include_special_with_doc = False
numpydoc_show_inherited_class_members = False
numpydoc_class_members_toctree = False

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
# source_suffix = ['.rst', '.md']
source_suffix = ".rst"

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = "index"

# General information about the project.
project = "PyLaTeX"
copyright = "2015, Jelte Fennema"
author = "Jelte Fennema"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = __version__.rstrip(".dirty")
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

autodoc_member_order = "bysource"
autodoc_default_flags = ["inherited-members"]
autoclass_content = "both"


def auto_change_docstring(app, what, name, obj, options, lines):
    r"""Make some automatic changes to docstrings.

    Things this function does are:

        - Add a title to module docstrings
        - Merge lines that end with a '\' with the next line.
    """
    if what == "module" and name.startswith("pylatex"):
        lines.insert(0, len(name) * "=")
        lines.insert(0, name)

    hits = 0
    for i, line in enumerate(lines.copy()):
        if line.endswith("\\"):
            lines[i - hits] += lines.pop(i + 1 - hits)
            hits += 1


def autodoc_allow_most_inheritance(app, what, name, obj, namespace, skip, options):
    cls = namespace.split(".")[-1]

    members = {
        "object": [
            "dump",
            "dumps_packages",
            "dump_packages",
            "latex_name",
            "escape",
            "generate_tex",
            "packages",
            "dumps_as_content",
            "end_paragraph",
            "separate_paragraph",
            "content_separator",
        ],
        "container": ["create", "dumps", "dumps_content", "begin_paragraph"],
        "userlist": [
            "append",
            "clear",
            "copy",
            "count",
            "extend",
            "index",
            "insert",
            "pop",
            "remove",
            "reverse",
            "sort",
        ],
        "error": ["args", "with_traceback"],
    }

    members["all"] = list(set([req for reqs in members.values() for req in reqs]))

    if name in members["all"]:
        skip = True

        if cls == "LatexObject":
            return False

        if cls in ("Container", "Environment") and name in members["container"]:
            return False

        if cls == "Document" and name == "generate_tex":
            return False

    if name == "separate_paragraph" and cls in ("SubFigure", "Float"):
        return False

    # Ignore all functions of NoEscape, since it is inherited
    if cls == "NoEscape":
        return True

    return skip


def setup(app):
    """Connect autodoc event to custom handler."""
    app.connect("autodoc-process-docstring", auto_change_docstring)
    app.connect("autodoc-skip-member", autodoc_allow_most_inheritance)


def linkcode_resolve(domain, info):
    """A simple function to find matching source code."""
    module_name = info["module"]
    fullname = info["fullname"]
    attribute_name = fullname.split(".")[-1]
    base_url = "https://github.com/JelteF/PyLaTeX/"

    if "+" in version:
        commit_hash = version.split(".")[-1][1:]
        base_url += "tree/%s/" % commit_hash
    else:
        base_url += "blob/v%s/" % version

    filename = module_name.replace(".", "/") + ".py"
    module = sys.modules.get(module_name)

    # Get the actual object
    try:
        actual_object = module
        for obj in fullname.split("."):
            parent = actual_object
            actual_object = getattr(actual_object, obj)
    except AttributeError:
        return None

    # Fix property methods by using their getter method
    if isinstance(actual_object, property):
        actual_object = actual_object.fget

    # Try to get the linenumber of the object
    try:
        source, start_line = inspect.getsourcelines(actual_object)
    except TypeError:
        # If it can not be found, try to find it anyway in the parents its
        # source code
        parent_source, parent_start_line = inspect.getsourcelines(parent)
        for i, line in enumerate(parent_source):
            if line.strip().startswith(attribute_name):
                start_line = parent_start_line + i
                end_line = start_line
                break
        else:
            return None

    else:
        end_line = start_line + len(source) - 1

    line_anchor = "#L%d-L%d" % (start_line, end_line)

    return base_url + filename + line_anchor


# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
default_role = "py:obj"

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = False

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# A list of ignored prefixes for module index sorting.
modindex_common_prefix = ["pylatex."]

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "matplotlib": ("http://matplotlib.org/", None),
    "numpy": ("https://docs.scipy.org/doc/numpy/", None),
    "quantities": ("https://pythonhosted.org/quantities/", "quantities-inv.txt"),
}


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = "sphinx_rtd_theme"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
html_favicon = "_static/realfavicongenerator.ico"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
# html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'h', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'r', 'sv', 'tr'
# html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# Now only 'ja' uses this config value
# html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
# html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.
htmlhelp_basename = "PyLaTeXdoc"

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
    # Latex figure (float) alignment
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, "PyLaTeX.tex", "PyLaTeX Documentation", "Jelte Fennema", "manual"),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [(master_doc, "pylatex", "PyLaTeX Documentation", [author], 1)]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "PyLaTeX",
        "PyLaTeX Documentation",
        author,
        "PyLaTeX",
        "One line description of project.",
        "Miscellaneous",
    ),
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
# texinfo_no_detailmenu = False



from pylatex import Command, Document, Section, Subsection


def test():
    doc = Document()
    Subsection("Only a single string", data="Some words")

    sec1 = Section("Only contains one subsection", data="Subsection")

    sec2 = Section("Only a single italic command", data=Command("textit", "Hey"))
    sec2.append("something else that is not italic")
    doc.append(sec1)
    doc.append(sec2)

    doc.generate_pdf()



#!/usr/bin/env python

import os

from pylatex import Document, Section
from pylatex.figure import Figure


def test():
    doc = Document()
    section = Section("Multirow Test")
    figure = Figure()
    image_filename = os.path.join(os.path.dirname(__file__), "../examples/kitten.jpg")
    figure.add_image(image_filename)
    figure.add_caption("Whoooo an imagage of a pdf")
    section.append(figure)
    doc.append(section)

    doc.generate_pdf()



import unittest

from pylatex import Document


class TestInheritance(unittest.TestCase):
    def test_latex_name(self):
        class MyDoc(Document):
            def __init__(self):
                super().__init__()

        doc = Document()
        my_doc = MyDoc()
        self.assertEqual(my_doc.latex_name, doc.latex_name)



# -*- coding: utf-8 -*-
import quantities as pq

from pylatex.quantities import Quantity, _dimensionality_to_siunitx


def test_quantity():
    v = 1 * pq.m / pq.s

    q1 = Quantity(v)
    assert q1.dumps() == r"\SI{1.0}{\meter\per\second}"

    q2 = Quantity(v, format_cb=lambda x: str(int(x)))
    assert q2.dumps() == r"\SI{1}{\meter\per\second}"

    q3 = Quantity(v, options={"zero-decimal-to-integer": "true"})
    ref = r"\SI[zero-decimal-to-integer=true]{1.0}{\meter\per\second}"
    assert q3.dumps() == ref


def test_quantity_float():
    q1 = Quantity(42.0)
    assert q1.dumps() == r"\num{42.0}"


def test_quantity_uncertain():
    t = pq.UncertainQuantity(7.0, pq.second, 1.0)
    q1 = Quantity(t)
    assert q1.dumps() == r"\SI{7.0 +- 1.0}{\second}"


def test_dimensionality_to_siunitx():
    assert (
        _dimensionality_to_siunitx((pq.volt / pq.kelvin).dimensionality)
        == r"\volt\per\Kelvin"
    )


if __name__ == "__main__":
    test_quantity()
    test_quantity_uncertain()
    test_dimensionality_to_siunitx()



# -*- coding: utf-8 -*-
"""
This module implements several classes that represent basic latex commands.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

from .base_classes import CommandBase, ContainerCommand, Environment
from .package import Package


class NewPage(CommandBase):
    """A command that adds a new page to the document."""


class LineBreak(NewPage):
    """A command that adds a line break to the document."""


class NewLine(NewPage):
    """A command that adds a new line to the document."""


class HFill(NewPage):
    """A command that fills the current line in the document."""


class HugeText(Environment):
    """An environment which makes the text size 'Huge'."""

    _latex_name = "Huge"

    def __init__(self, data=None):
        """
        Args
        ----
        data : str or `~.LatexObject`
            The string or LatexObject to be formatted.
        """

        super().__init__(data=data)


class LargeText(HugeText):
    """An environment which makes the text size 'Large'."""

    _latex_name = "Large"


class MediumText(HugeText):
    """An environment which makes the text size 'large'."""

    _latex_name = "large"


class SmallText(HugeText):
    """An environment which makes the text size 'small'."""

    _latex_name = "small"


class FootnoteText(HugeText):
    """An environment which makes the text size 'footnotesize'."""

    _latex_name = "footnotesize"


class TextColor(ContainerCommand):
    """An environment which changes the text color of the data."""

    _repr_attributes_mapping = {"color": "arguments"}

    packages = [Package("xcolor")]

    def __init__(self, color, data):
        """
        Args
        ----
        color: str
            The color to set for the data inside of the environment.
        data: str or `~.LatexObject`
            The string or LatexObject to be formatted.
        """

        super().__init__(arguments=color, data=data)



# -*- coding: utf-8 -*-
r"""A test to make sure the document compiles with inputenc set to `None`."""

from pylatex import Document
from pylatex.base_classes import Arguments

doc = Document("no_inputenc", inputenc=None)
doc.append("test text")

# Make sure inputenc isn't used
assert not any([p.arguments == Arguments("inputenc") for p in doc.packages])

doc.generate_pdf(clean=True, clean_tex=False, silent=False)



#!/usr/bin/python
"""
This example shows how multirow and multicolumns can be used.

..  :copyright: (c) 2014 by Jelte Fennema.
    :license: MIT, see License for more details.
"""

# begin-doc-include
from pylatex import Document, MultiColumn, MultiRow, Section, Subsection, Tabular

doc = Document("multirow")
section = Section("Multirow Test")

test1 = Subsection("MultiColumn")
test2 = Subsection("MultiRow")
test3 = Subsection("MultiColumn and MultiRow")
test4 = Subsection("Vext01")

table1 = Tabular("|c|c|c|c|")
table1.add_hline()
table1.add_row((MultiColumn(4, align="|c|", data="Multicolumn"),))
table1.add_hline()
table1.add_row((1, 2, 3, 4))
table1.add_hline()
table1.add_row((5, 6, 7, 8))
table1.add_hline()
row_cells = ("9", MultiColumn(3, align="|c|", data="Multicolumn not on left"))
table1.add_row(row_cells)
table1.add_hline()

table2 = Tabular("|c|c|c|")
table2.add_hline()
table2.add_row((MultiRow(3, data="Multirow"), 1, 2))
table2.add_hline(2, 3)
table2.add_row(("", 3, 4))
table2.add_hline(2, 3)
table2.add_row(("", 5, 6))
table2.add_hline()
table2.add_row((MultiRow(3, data="Multirow2"), "", ""))
table2.add_empty_row()
table2.add_empty_row()
table2.add_hline()

table3 = Tabular("|c|c|c|")
table3.add_hline()
table3.add_row(
    (MultiColumn(2, align="|c|", data=MultiRow(2, data="multi-col-row")), "X")
)
table3.add_row((MultiColumn(2, align="|c|", data=""), "X"))
table3.add_hline()
table3.add_row(("X", "X", "X"))
table3.add_hline()

table4 = Tabular("|c|c|c|")
table4.add_hline()
col1_cell = MultiRow(4, data="span-4")
col2_cell = MultiRow(2, data="span-2")
table4.add_row((col1_cell, col2_cell, "3a"))
table4.add_hline(start=3)
table4.add_row(("", "", "3b"))
table4.add_hline(start=2)
table4.add_row(("", col2_cell, "3c"))
table4.add_hline(start=3)
table4.add_row(("", "", "3d"))
table4.add_hline()

test1.append(table1)
test2.append(table2)
test3.append(table3)
test4.append(table4)

section.append(test1)
section.append(test2)
section.append(test3)
section.append(test4)

doc.append(section)
doc.generate_pdf(clean_tex=False)



#!/usr/bin/env python

import os
import shutil

from pylatex import Document


def test():
    doc = Document("jobname_test", data=["Jobname test"])
    doc.generate_pdf()

    assert os.path.isfile("jobname_test.pdf")

    os.remove("jobname_test.pdf")

    folder = "tmp_jobname"
    os.makedirs(folder)
    path = os.path.join(folder, "jobname_test_dir")

    doc = Document(path, data=["Jobname test dir"])
    doc.generate_pdf()

    assert os.path.isfile(path + ".pdf")
    shutil.rmtree(folder)

    folder = "tmp_jobname2"
    os.makedirs(folder)
    path = os.path.join(folder, "jobname_test_dir2")

    doc = Document(path, data=["Jobname test dir"])
    doc.generate_pdf(os.path.join(folder, ""))

    assert os.path.isfile(path + ".pdf")

    shutil.rmtree(folder)
